from django.contrib import messages
from django.contrib.auth.mixins import LoginRequiredMixin
from django.contrib.auth.models import Group
from django.shortcuts import redirect
from django.urls import reverse
from django.utils.translation import gettext as _
from django.views.generic import (
    CreateView,
    DeleteView,
    DetailView,
    ListView,
    TemplateView,
    UpdateView,
)
from django.views.generic.detail import SingleObjectMixin
from guardian.shortcuts import get_objects_for_group

from ephios.extra.permissions import CustomPermissionRequiredMixin
from ephios.user_management import mail
from ephios.user_management.forms import GroupForm, QualificationGrantFormset, UserProfileForm
from ephios.user_management.models import UserProfile


class ProfileView(LoginRequiredMixin, DetailView):
    def get_object(self, queryset=None):
        return self.request.user


class UserProfileListView(CustomPermissionRequiredMixin, ListView):
    model = UserProfile
    permission_required = "user_management.view_userprofile"


class UserProfileCreateView(CustomPermissionRequiredMixin, TemplateView):
    template_name = "user_management/userprofile_form.html"
    permission_required = "user_management.add_userprofile"
    model = UserProfile

    def get_context_data(self, **kwargs):
        kwargs.setdefault("userprofile_form", UserProfileForm(self.request.POST or None))
        kwargs.setdefault(
            "qualification_formset", QualificationGrantFormset(self.request.POST or None)
        )
        return super().get_context_data(**kwargs)

    def post(self, request, *args, **kwargs):
        userprofile_form = UserProfileForm(self.request.POST)
        qualification_formset = QualificationGrantFormset(self.request.POST)
        if all((userprofile_form.is_valid(), qualification_formset.is_valid())):
            userprofile = userprofile_form.save()
            qualification_formset.instance = userprofile
            qualification_formset.save()
            messages.success(
                self.request,
                _("User {name} ({user}) added successfully.").format(
                    name=userprofile.get_full_name(), user=userprofile
                ),
            )
            if userprofile.is_active:
                mail.send_account_creation_info(userprofile)
            return redirect(reverse("user_management:userprofile_list"))
        else:
            return self.render_to_response(
                self.get_context_data(
                    userprofile_form=userprofile_form, qualification_formset=qualification_formset
                )
            )


class UserProfileUpdateView(CustomPermissionRequiredMixin, SingleObjectMixin, TemplateView):
    model = UserProfile
    permission_required = "user_management.change_userprofile"
    template_name = "user_management/userprofile_form.html"

    def get_userprofile_form(self):
        return UserProfileForm(
            self.request.POST or None,
            initial={
                "groups": self.get_object().groups.all(),
            },
            instance=self.object,
        )

    def get_qualification_formset(self):
        return QualificationGrantFormset(self.request.POST or None, instance=self.object)

    def get_context_data(self, **kwargs):
        self.object = self.get_object()
        kwargs.setdefault("userprofile_form", self.get_userprofile_form())
        kwargs.setdefault("qualification_formset", self.get_qualification_formset())
        return super().get_context_data(**kwargs)

    def post(self, request, *args, **kwargs):
        self.object = self.get_object()
        userprofile_form = self.get_userprofile_form()
        qualification_formset = self.get_qualification_formset()
        if all((userprofile_form.is_valid(), qualification_formset.is_valid())):
            userprofile = userprofile_form.save()
            qualification_formset.save()
            messages.success(
                self.request,
                _("User {name} ({user}) updated successfully.").format(
                    name=self.object.get_full_name(), user=self.object
                ),
            )
            if userprofile.is_active:
                mail.send_account_update_info(userprofile)
            return redirect(reverse("user_management:userprofile_list"))

        return self.render_to_response(
            self.get_context_data(
                userprofile_form=userprofile_form, qualification_formset=qualification_formset
            )
        )


class UserProfileDeleteView(CustomPermissionRequiredMixin, DeleteView):
    model = UserProfile
    permission_required = "user_management.delete_userprofile"
    template_name = "user_management/userprofile_confirm_delete.html"

    def get_success_url(self):
        messages.info(
            self.request,
            _("The user {name} ({user}) was deleted.").format(
                name=self.object.get_full_name(), user=self.object
            ),
        )
        return reverse("user_management:userprofile_list")


class GroupListView(CustomPermissionRequiredMixin, ListView):
    model = Group
    permission_required = "auth.view_group"
    template_name = "user_management/group_list.html"


class GroupCreateView(CustomPermissionRequiredMixin, CreateView):
    model = Group
    permission_required = "auth.add_group"
    template_name = "user_management/group_form.html"
    form_class = GroupForm

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs["initial"] = {
            "users": UserProfile.objects.none(),
            "can_add_event": False,
            "publish_event_for_group": Group.objects.none(),
        }
        return kwargs

    def get_success_url(self):
        messages.success(
            self.request, _('Group "{group}" created successfully.').format(group=self.object)
        )
        return reverse("user_management:group_list")


class GroupUpdateView(CustomPermissionRequiredMixin, UpdateView):
    model = Group
    permission_required = "auth.change_group"
    template_name = "user_management/group_form.html"
    form_class = GroupForm

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs["initial"] = {
            "users": self.object.user_set.all(),
            "can_view_past_event": self.object.permissions.filter(
                codename="view_past_event"
            ).exists(),
            "can_add_event": self.object.permissions.filter(codename="add_event").exists(),
            "publish_event_for_group": get_objects_for_group(
                self.object, "publish_event_for_group", klass=Group
            ),
        }
        return kwargs

    def get_success_url(self):
        messages.success(
            self.request, _('Group "{group}" updated successfully.').format(group=self.object)
        )
        return reverse("user_management:group_list")


class GroupDeleteView(CustomPermissionRequiredMixin, DeleteView):
    model = Group
    permission_required = "auth.delete_group"
    template_name = "user_management/group_confirm_delete.html"

    def get_success_url(self):
        messages.info(self.request, _('The group "{group}" was deleted.').format(group=self.object))
        return reverse("user_management:group_list")



from django.urls import path

from ephios.user_management import views

app_name = "user_management"
urlpatterns = [
    path("profile/", views.ProfileView.as_view(), name="profile"),
    path("groups/", views.GroupListView.as_view(), name="group_list"),
    path("groups/<int:pk>/edit", views.GroupUpdateView.as_view(), name="group_edit"),
    path("groups/<int:pk>/delete", views.GroupDeleteView.as_view(), name="group_delete"),
    path("groups/create", views.GroupCreateView.as_view(), name="group_add"),
    path(
        "users/",
        views.UserProfileListView.as_view(),
        name="userprofile_list",
    ),
    path(
        "users/<int:pk>/edit",
        views.UserProfileUpdateView.as_view(),
        name="userprofile_edit",
    ),
    path(
        "users/<int:pk>/delete",
        views.UserProfileDeleteView.as_view(),
        name="userprofile_delete",
    ),
    path(
        "users/create/",
        views.UserProfileCreateView.as_view(),
        name="userprofile_create",
    ),
]



from django import template
from django.utils.safestring import mark_safe

from ephios.event_management.models import AbstractParticipation

register = template.Library()


@register.filter(name="shift_status")
def shift_status(shift, user):
    participation = user.as_participant().participation_for(shift)
    if participation is not None:
        color = {
            AbstractParticipation.States.USER_DECLINED: "text-danger",
            AbstractParticipation.States.RESPONSIBLE_REJECTED: "text-danger",
            AbstractParticipation.States.REQUESTED: "text-warning",
            AbstractParticipation.States.CONFIRMED: "text-success",
        }[participation.state]
        return mark_safe(f'<span class="{color}">{participation.get_state_display()}</span><br>')
    return ""


@register.filter(name="can_sign_up")
def can_sign_up(shift, user):
    return shift.signup_method.can_sign_up(user.as_participant())


@register.filter(name="render_shift_state")
def render_shift_state(shift, request):
    return shift.signup_method.render_shift_state(request)


@register.filter(name="signup_errors")
def signup_errors(shift, user):
    return shift.signup_method.get_signup_errors(user.as_participant())


@register.filter(name="can_decline")
def can_decline(shift, user):
    return shift.signup_method.can_decline(user.as_participant())


@register.filter(name="decline_errors")
def decline_errors(shift, user):
    return shift.signup_method.get_decline_errors(user.as_participant())



from django.contrib import messages
from django.contrib.auth import get_user_model
from django.contrib.auth.mixins import LoginRequiredMixin
from django.contrib.auth.models import Group
from django.core.exceptions import ValidationError
from django.db.models import Max, Min
from django.http import HttpResponse
from django.shortcuts import get_object_or_404, redirect
from django.urls import reverse, reverse_lazy
from django.utils import timezone
from django.utils.timezone import get_default_timezone
from django.utils.translation import gettext as _
from django.views.generic import (
    CreateView,
    DeleteView,
    DetailView,
    ListView,
    TemplateView,
    UpdateView,
    View,
)
from django.views.generic.detail import SingleObjectMixin
from guardian.shortcuts import get_objects_for_user, get_users_with_perms

from ephios.event_management.forms import EventForm, ShiftForm
from ephios.event_management.models import Event, Shift
from ephios.extra.permissions import CustomPermissionRequiredMixin, get_groups_with_perms


class HomeView(LoginRequiredMixin, TemplateView):
    template_name = "event_management/home.html"


class EventListView(LoginRequiredMixin, ListView):
    model = Event

    def get_queryset(self):
        return (
            get_objects_for_user(self.request.user, "event_management.view_event")
            .annotate(
                start_time=Min("shifts__start_time"),
                end_time=Max("shifts__end_time"),
            )
            .filter(end_time__gte=timezone.now())
            .select_related("type")
        )


class EventDetailView(CustomPermissionRequiredMixin, DetailView):
    model = Event
    permission_required = "event_management.view_event"

    def get_queryset(self):
        if self.request.user.has_perm("event_management.add_event"):
            return Event.all_objects.all()
        else:
            return Event.objects.all()


class EventUpdateView(CustomPermissionRequiredMixin, UpdateView):
    model = Event
    queryset = Event.all_objects.all()
    permission_required = "event_management.change_event"

    def get_form(self, form_class=None):
        visible_queryset = get_objects_for_user(
            self.request.user, "publish_event_for_group", klass=Group
        )
        visible_initial = get_groups_with_perms(self.object, only_with_perms_in=["view_event"])
        initial = {
            "visible_for": visible_initial,
            "responsible_persons": get_users_with_perms(
                self.object, only_with_perms_in=["change_event"], with_group_users=False
            ),
            "responsible_groups": get_groups_with_perms(
                self.object, only_with_perms_in=["change_event"]
            ),
        }
        event_form = EventForm(self.request.POST or None, instance=self.object, initial=initial)
        event_form.fields["visible_for"].queryset = visible_queryset
        event_form.fields["visible_for"].disabled = not visible_queryset
        groups_without_perms = visible_initial.exclude(id__in=visible_queryset)
        if groups_without_perms:
            event_form.fields["visible_for"].help_text = _(
                "Select groups which the event shall be visible for. "
                "This event is also visible for <b>{groups}</b>, "
                "but you don't have the permission to change visibility "
                "for those groups."
            ).format(groups=", ".join(groups_without_perms.values_list("name", flat=True)))
        return event_form


class EventCreateView(CustomPermissionRequiredMixin, CreateView):
    template_name = "event_management/event_form.html"
    permission_required = "event_management.add_event"

    def get_form(self, form_class=None):
        visible_for_queryset = get_objects_for_user(
            self.request.user, "publish_event_for_group", klass=Group
        )
        event_form = EventForm(
            self.request.POST or None,
            initial={
                "responsible_persons": get_user_model().objects.filter(pk=self.request.user.pk),
                "responsible_groups": Group.objects.none(),
                "visible_for": visible_for_queryset,
            },
        )
        event_form.fields["visible_for"].queryset = visible_for_queryset
        return event_form

    def get_context_data(self, **kwargs):
        inactive_events = Event.all_objects.filter(active=False)
        kwargs.setdefault("inactive_events", inactive_events)
        return super().get_context_data(**kwargs)

    def get_success_url(self):
        return reverse("event_management:event_createshift", kwargs={"pk": self.object.pk})


class EventActivateView(CustomPermissionRequiredMixin, SingleObjectMixin, View):
    permission_required = "event_management.add_event"
    queryset = Event.all_objects.all()

    def post(self, request, *args, **kwargs):
        event = self.get_object()
        try:
            event.activate()
            messages.success(
                request,
                _("The event {title} has been saved.").format(title=event.title),
            )
        except ValidationError as e:
            messages.error(request, e)
        return redirect(reverse("event_management:event_detail", kwargs={"pk": event.pk}))


class EventDeleteView(CustomPermissionRequiredMixin, DeleteView):
    queryset = Event.all_objects.all()
    permission_required = "event_management.delete_event"
    success_url = reverse_lazy("event_management:event_list")


class EventArchiveView(CustomPermissionRequiredMixin, ListView):
    permission_required = "event_management.view_past_event"
    model = Event
    template_name = "event_management/event_archive.html"

    def get_queryset(self):
        return (
            get_objects_for_user(self.request.user, "event_management.view_event")
            .annotate(
                start_time=Min("shifts__start_time"),
                end_time=Max("shifts__end_time"),
            )
            .filter(end_time__lt=timezone.now())
            .select_related("type")
        )


class ShiftCreateView(CustomPermissionRequiredMixin, TemplateView):
    permission_required = "event_management.add_event"
    template_name = "event_management/shift_form.html"

    def get_event(self):
        return get_object_or_404(Event.all_objects, pk=self.kwargs.get("pk"))

    def get_shift_form(self):
        return ShiftForm(self.request.POST or None)

    def get_context_data(self, **kwargs):
        event = self.get_event()
        kwargs.setdefault("event", event)
        kwargs.setdefault("form", self.get_shift_form())
        return super().get_context_data(**kwargs)

    def post(self, *args, **kwargs):
        form = self.get_shift_form()
        try:
            from ephios.event_management.signup import signup_method_from_slug

            signup_method = signup_method_from_slug(self.request.POST["signup_method_slug"])
            configuration_form = signup_method.get_configuration_form(self.request.POST)
        except ValueError as e:
            raise ValidationError(e)
        if form.is_valid() and configuration_form.is_valid():
            shift = form.save(commit=False)
            event = self.get_event()
            shift.event = event
            shift.signup_configuration = configuration_form.get_configuration()
            shift.save()
            if "addAnother" in self.request.POST:
                return redirect(
                    reverse(
                        "event_management:event_createshift", kwargs={"pk": self.kwargs.get("pk")}
                    )
                )
            else:
                try:
                    event.activate()
                    messages.success(
                        self.request,
                        _("The event {title} has been saved.").format(title=event.title),
                    )
                except ValidationError as e:
                    messages.error(self.request, e)
                return redirect(event.get_absolute_url())
        else:
            return self.render_to_response(
                self.get_context_data(
                    form=form,
                    configuration_form=signup_method.render_configuration_form(
                        form=configuration_form
                    ),
                )
            )


class ShiftConfigurationFormView(View):
    def get(self, request, *args, **kwargs):
        from ephios.event_management.signup import signup_method_from_slug

        signup_method = signup_method_from_slug(self.kwargs.get("slug"))
        return HttpResponse(signup_method.render_configuration_form())


class ShiftUpdateView(CustomPermissionRequiredMixin, SingleObjectMixin, TemplateView):
    model = Shift
    template_name = "event_management/shift_form.html"
    permission_required = "event_management.change_event"

    def get_permission_object(self):
        return self.get_object().event

    def get_shift_form(self):
        return ShiftForm(
            self.request.POST or None,
            instance=self.object,
            initial={
                "date": self.object.meeting_time.date(),
                "meeting_time": self.object.meeting_time.astimezone(get_default_timezone()).time(),
                "start_time": self.object.start_time.astimezone(get_default_timezone()).time(),
                "end_time": self.object.end_time.astimezone(get_default_timezone()).time(),
            },
        )

    def get_configuration_form(self):
        return self.object.signup_method.render_configuration_form(data=self.request.POST or None)

    def get_context_data(self, **kwargs):
        self.object = self.get_object()
        kwargs.setdefault("event", self.object.event)
        kwargs.setdefault("form", self.get_shift_form())
        kwargs.setdefault("configuration_form", self.get_configuration_form())
        return super().get_context_data(**kwargs)

    def post(self, *args, **kwargs):
        self.object = self.get_object()
        form = self.get_shift_form()
        try:
            from ephios.event_management.signup import signup_method_from_slug

            signup_method = signup_method_from_slug(self.request.POST["signup_method_slug"])
            configuration_form = signup_method.get_configuration_form(self.request.POST)
        except ValueError as e:
            raise ValidationError(e)
        if form.is_valid() and configuration_form.is_valid():
            shift = form.save(commit=False)
            shift.signup_configuration = configuration_form.get_configuration()
            shift.save()
            if "addAnother" in self.request.POST:
                return redirect(
                    reverse("event_management:event_createshift", kwargs={"pk": shift.event.pk})
                )
            else:
                messages.success(
                    self.request, _("The shift {shift} has been saved.").format(shift=shift)
                )
                return redirect(self.object.event.get_absolute_url())
        else:
            return self.render_to_response(
                self.get_context_data(
                    form=form,
                    configuration_form=signup_method.render_configuration_form(
                        form=configuration_form
                    ),
                )
            )


class ShiftDeleteView(CustomPermissionRequiredMixin, DeleteView):
    permission_required = "event_management.change_event"
    model = Shift

    def setup(self, request, *args, **kwargs):
        super().setup(request, *args, **kwargs)
        self.object = self.get_object()

    def delete(self, request, *args, **kwargs):
        if self.object.event.shifts.count() == 1:
            messages.error(self.request, _("You cannot delete the last shift!"))
            return redirect(self.object.event.get_absolute_url())
        else:
            return super().delete(request, *args, **kwargs)

    def get_success_url(self):
        messages.success(self.request, _("The shift has been deleted."))
        return self.object.event.get_absolute_url()

    def get_permission_object(self):
        return self.object.event


class SignupMethodViewMixin(SingleObjectMixin):
    model = Shift

    def dispatch(self, request, *args, **kwargs):
        return self.get_object().signup_method.signup_view(request, *args, **kwargs)


class ShiftSignupView(CustomPermissionRequiredMixin, SignupMethodViewMixin, View):
    permission_required = "event_management.view_event"

    def get_permission_object(self):
        return self.get_object().event



from django.dispatch import receiver

from ephios.event_management.signup import register_signup_methods
from ephios.plugins.basesignup.signup.confirm import RequestConfirmSignupMethod
from ephios.plugins.basesignup.signup.instant import InstantConfirmationSignupMethod


@receiver(
    register_signup_methods,
    dispatch_uid="ephios.plugins.basesignup.signals.register_signup_methods",
)
def register_signup_methods(sender, **kwargs):
    return [InstantConfirmationSignupMethod, RequestConfirmSignupMethod]



from django.urls import path

from ephios.plugins.basesignup.signup.confirm import RequestConfirmDispositionView

app_name = "basesignup"
urlpatterns = [
    path(
        "shifts/<int:pk>/disposition/requestconfirm",
        RequestConfirmDispositionView.as_view(),
        name="shift_disposition_requestconfirm",
    ),
]



# Generated by Django 3.1.1 on 2020-09-21 19:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("event_management", "0008_event_mail_updates"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="event",
            options={
                "permissions": [("view_past_event", "Can view past events")],
                "verbose_name": "event",
                "verbose_name_plural": "events",
            },
        ),
        migrations.AlterField(
            model_name="shift",
            name="signup_configuration",
            field=models.JSONField(default=dict),
        ),
    ]



# Generated by Django 3.1.1 on 2020-09-22 20:43

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Page",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("title", models.CharField(max_length=250)),
                ("content", models.TextField(blank=True)),
                ("slug", models.SlugField(max_length=250, unique=True)),
                ("show_in_footer", models.BooleanField(default=False)),
                ("publicly_visible", models.BooleanField(default=False)),
            ],
        ),
    ]



from django import forms
from django.contrib.auth.forms import ReadOnlyPasswordHashField
from django.contrib.auth.models import Group
from django.contrib.auth.password_validation import validate_password
from django.forms import (
    BooleanField,
    ModelForm,
    ModelMultipleChoiceField,
    TextInput,
    inlineformset_factory,
)
from django.utils.translation import gettext as _
from django_select2.forms import Select2MultipleWidget, Select2Widget
from guardian.shortcuts import assign_perm, remove_perm

from ephios.extra.widgets import CustomDateInput
from ephios.user_management.models import QualificationGrant, UserProfile
from ephios.user_management.widgets import MultiUserProfileWidget


class UserCreationForm(forms.ModelForm):
    """A form for creating new users. Includes all the required
    fields, plus a repeated password."""

    password = forms.CharField(label=_("Password"), widget=forms.PasswordInput)
    password_validation = forms.CharField(
        label=_("Password confirmation"), widget=forms.PasswordInput
    )
    field_order = ["email", "password", "password_validation"]

    class Meta:
        model = UserProfile
        fields = (
            "email",
            "first_name",
            "last_name",
            "date_of_birth",
            "phone",
        )

    def clean_password_validation(self):
        # Check that the two password entries match
        password = self.cleaned_data.get("password")
        password_validation = self.cleaned_data.get("password_validation")
        if password and password_validation and password != password_validation:
            raise forms.ValidationError(_("Passwords don't match"))
        return password_validation

    def _post_clean(self):
        super()._post_clean()
        # Validate the password after self.instance is updated with form data
        # by super().
        password = self.cleaned_data.get("password_validation")
        if password:
            try:
                validate_password(password, self.instance)
            except forms.ValidationError as error:
                self.add_error("password", error)

    def save(self, commit=True):
        # Save the provided password in hashed format
        user = super().save(commit=False)
        user.set_password(self.cleaned_data["password"])
        if commit:
            user.save()
        return user


class UserChangeForm(forms.ModelForm):
    """A form for updating users. Includes all the fields on
    the user, but replaces the password field with admin's
    password hash display field.
    """

    password = ReadOnlyPasswordHashField()

    class Meta:
        model = UserProfile
        fields = (
            "email",
            "password",
            "first_name",
            "last_name",
            "date_of_birth",
            "phone",
            "is_active",
            "is_staff",
        )

    def clean_password(self):
        # Regardless of what the user provides, return the initial value.
        # This is done here, rather than on the field, because the
        # field does not have access to the initial value
        return self.initial["password"]


class GroupForm(ModelForm):
    publish_event_for_group = ModelMultipleChoiceField(
        label=_("Can publish event for group"),
        queryset=Group.objects.all(),
        required=False,
        help_text=_("Choose groups that this group can make events visible for."),
        widget=Select2MultipleWidget,
    )
    can_view_past_event = BooleanField(label=_("Can view past events"), required=False)
    can_add_event = BooleanField(label=_("Can add event"), required=False)
    users = ModelMultipleChoiceField(
        label=_("Users"), queryset=UserProfile.objects.all(), widget=MultiUserProfileWidget
    )

    field_order = [
        "name",
        "users",
        "can_view_past_event",
        "can_add_event",
        "publish_event_for_group",
    ]

    class Meta:
        model = Group
        fields = ["name"]

    def save(self, commit=True):
        group = super().save(commit)

        group.user_set.set(self.cleaned_data["users"])

        if self.cleaned_data["can_view_past_event"]:
            assign_perm("event_management.view_past_event", group)
        else:
            remove_perm("event_management.view_past_event", group)

        remove_perm("publish_event_for_group", group, Group.objects.all())
        if self.cleaned_data["can_add_event"]:
            assign_perm("event_management.add_event", group)
            assign_perm("event_management.delete_event", group)
            assign_perm(
                "publish_event_for_group", group, self.cleaned_data["publish_event_for_group"]
            )
        else:
            remove_perm("event_management.add_event", group)
            remove_perm("event_management.delete_event", group)

        return group


class UserProfileForm(ModelForm):
    groups = ModelMultipleChoiceField(
        label=_("Groups"), queryset=Group.objects.all(), widget=Select2MultipleWidget
    )

    field_order = [
        "email",
        "first_name",
        "last_name",
        "date_of_birth",
        "phone",
        "groups",
        "is_active",
    ]

    class Meta:
        model = UserProfile
        fields = ["email", "first_name", "last_name", "date_of_birth", "phone", "is_active"]
        widgets = {"date_of_birth": CustomDateInput(format="%Y-%m-%d")}
        help_texts = {
            "is_active": _("Inactive users cannot log in and do not get any notifications.")
        }
        labels = {"is_active": _("Active user")}

    def save(self, commit=True):
        userprofile = super().save(commit)
        userprofile.groups.set(self.cleaned_data["groups"])
        userprofile.save()
        return userprofile


class QualificationGrantForm(ModelForm):
    model = QualificationGrant

    class Meta:
        fields = ["qualification", "expires"]
        widgets = {"qualification": Select2Widget, "expires": CustomDateInput(format="%Y-%m-%d")}

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        instance = getattr(self, "instance", None)
        if instance and instance.pk:
            # Hide the field and simply display the qualification name in the template
            self.fields["qualification"].disabled = True
            self.fields["qualification"].widget = forms.HiddenInput()
            self.fields["qualification"].title = instance.qualification.title


QualificationGrantFormset = inlineformset_factory(
    UserProfile,
    QualificationGrant,
    form=QualificationGrantForm,
    extra=0,
)



from django.db import models


class Page(models.Model):
    title = models.CharField(max_length=250)
    content = models.TextField(blank=True)
    slug = models.SlugField(max_length=250, unique=True)
    show_in_footer = models.BooleanField(default=False)
    publicly_visible = models.BooleanField(default=False)

    def __str__(self):
        return self.title



# Generated by Django 3.0.6 on 2020-08-29 13:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("user_management", "0004_auto_20200829_1535"),
    ]

    operations = [
        migrations.AddField(
            model_name="qualification",
            name="abbreviation",
            field=models.CharField(default="", max_length=254),
            preserve_default=False,
        ),
    ]



import functools
import json
from argparse import Namespace
from dataclasses import dataclass, field
from datetime import date
from typing import List, Optional

import django.dispatch
from django import forms
from django.contrib import messages
from django.contrib.auth import get_user_model
from django.core.exceptions import ValidationError
from django.db import transaction
from django.db.models import QuerySet
from django.shortcuts import redirect
from django.template import Context, Template
from django.template.defaultfilters import yesno
from django.utils import formats, timezone
from django.utils.functional import cached_property
from django.utils.translation import gettext_lazy as _
from django.views import View

from ephios.event_management.models import AbstractParticipation, LocalParticipation, Shift
from ephios.extra.json import CustomJSONDecoder, CustomJSONEncoder
from ephios.extra.widgets import CustomSplitDateTimeWidget
from ephios.user_management.models import Qualification

register_signup_methods = django.dispatch.Signal(providing_args=[])


def all_signup_methods():
    for _, methods in register_signup_methods.send(None):
        yield from methods


def signup_method_from_slug(slug, shift=None):
    for method in all_signup_methods():
        if method.slug == slug:
            return method(shift)
    raise ValueError(_("Signup Method '{slug}' was not found.").format(slug=slug))


@dataclass(frozen=True)
class AbstractParticipant:
    first_name: str
    last_name: str
    qualifications: QuerySet = field(hash=False)
    date_of_birth: date
    email: Optional[str]  # if set to None, no notifications are sent

    @property
    def age(self):
        today, born = date.today(), self.date_of_birth
        return today.year - born.year - ((today.month, today.day) < (born.month, born.day))

    def __str__(self):
        return f"{self.first_name} {self.last_name}"

    def new_participation(self, shift):
        raise NotImplementedError

    def participation_for(self, shift):
        """Return the participation object for a shift. Return None if it does not exist."""
        raise NotImplementedError

    def collect_all_qualifications(self):
        """We collect using breath first search with one query for every layer of inclusion."""
        all_qualifications = set(self.qualifications)
        current = self.qualifications
        while current:
            next = (
                Qualification.objects.filter(included_by__in=current)
                .exclude(id__in=(q.id for q in all_qualifications))
                .distinct()
            )
            all_qualifications |= set(next)
            current = next
        return all_qualifications

    def has_qualifications(self, qualifications):
        return set(qualifications) <= self.collect_all_qualifications()


@dataclass(frozen=True)
class LocalUserParticipant(AbstractParticipant):
    user: get_user_model()

    def new_participation(self, shift):
        return LocalParticipation(shift=shift, user=self.user)

    def participation_for(self, shift):
        try:
            return LocalParticipation.objects.get(shift=shift, user=self.user)
        except LocalParticipation.DoesNotExist:
            return None


class ParticipationError(ValidationError):
    pass


class ConfigurationForm(forms.Form):
    def get_configuration(self):
        return json.dumps(self.cleaned_data, cls=CustomJSONEncoder)


def check_event_is_active(method, participant):
    if not method.shift.event.active:
        return ParticipationError(_("The event is not active."))


def check_participation_state_for_signup(method, participant):
    participation = participant.participation_for(method.shift)
    if participation is not None:
        if participation.state == AbstractParticipation.States.REQUESTED:
            return ParticipationError(
                _("You have already requested your participation for {shift}").format(
                    shift=method.shift
                )
            )
        elif participation.state == AbstractParticipation.States.CONFIRMED:
            return ParticipationError(
                _("You are already signed up for {shift}.").format(shift=method.shift)
            )
        elif participation.state == AbstractParticipation.States.RESPONSIBLE_REJECTED:
            return ParticipationError(
                _("You are rejected from {shift}.").format(shift=method.shift)
            )


def check_participation_state_for_decline(method, participant):
    participation = participant.participation_for(method.shift)
    if participation is not None:
        if (
            participation.state == AbstractParticipation.States.CONFIRMED
            and not method.configuration.user_can_decline_confirmed
        ):
            return ParticipationError(
                _("You are bindingly signed up for {shift}.").format(shift=method.shift)
            )
        elif participation.state == AbstractParticipation.States.RESPONSIBLE_REJECTED:
            return ParticipationError(
                _("You are rejected from {shift}.").format(shift=method.shift)
            )
        elif participation.state == AbstractParticipation.States.USER_DECLINED:
            return ParticipationError(
                _("You have already declined participating in {shift}.").format(shift=method.shift)
            )


def check_inside_signup_timeframe(method, participant):
    last_time = method.shift.end_time
    if method.configuration.signup_until is not None:
        last_time = min(last_time, method.configuration.signup_until)
    if timezone.now() > last_time:
        return ParticipationError(_("The signup period is over."))


def check_participant_age(method, participant):
    minimum_age = method.configuration.minimum_age
    if minimum_age is not None and participant.age < minimum_age:
        return ParticipationError(
            _("You are too young. The minimum age is {age}.").format(age=minimum_age)
        )


class BaseSignupMethod:
    slug = "abstract"
    verbose_name = "abstract"
    description = """"""

    # use _ == gettext_lazy!
    registration_button_text = _("Sign up")
    signup_success_message = _("You have successfully signed up for {shift}.")
    signup_error_message = _("Signing up failed: {error}")
    decline_success_message = _("You have successfully declined {shift}.")
    decline_error_message = _("Declining failed: {error}")

    def __init__(self, shift):
        self.shift = shift
        self.configuration = Namespace(
            **{name: config["default"] for name, config in self.get_configuration_fields().items()}
        )
        if shift is not None:
            for key, value in json.loads(
                shift.signup_configuration if shift is not None else "{}", cls=CustomJSONDecoder
            ).items():
                setattr(self.configuration, key, value)

    @property
    def signup_view_class(self):
        return BaseSignupView

    @cached_property
    def signup_view(self):
        return self.signup_view_class.as_view(method=self, shift=self.shift)

    @property
    def signup_checkers(self):
        return [
            check_event_is_active,
            check_participation_state_for_signup,
            check_inside_signup_timeframe,
            check_participant_age,
        ]

    @property
    def decline_checkers(self):
        return [
            check_event_is_active,
            check_participation_state_for_decline,
            check_inside_signup_timeframe,
        ]

    @functools.lru_cache()
    def get_signup_errors(self, participant) -> List[ParticipationError]:
        return [
            error
            for checker in self.signup_checkers
            if (error := checker(self, participant)) is not None
        ]

    @functools.lru_cache()
    def get_decline_errors(self, participant):
        return [
            error
            for checker in self.decline_checkers
            if (error := checker(self, participant)) is not None
        ]

    def can_decline(self, participant):
        return not self.get_decline_errors(participant)

    def can_sign_up(self, participant):
        return not self.get_signup_errors(participant)

    def get_participation_for(self, participant):
        return participant.participation_for(self.shift) or participant.new_participation(
            self.shift
        )

    def perform_signup(self, participant: AbstractParticipant, **kwargs):
        """
        Configure a participation object for the given participant according to the method's configuration.
        `kwargs` may contain further instructions from a e.g. a form.
        """
        if errors := self.get_signup_errors(participant):
            raise ParticipationError(errors)
        return self.get_participation_for(participant)

    def perform_decline(self, participant, **kwargs):
        """Create and configure a declining participation object for the given participant. `kwargs` may contain further instructions from a e.g. a form."""
        if errors := self.get_decline_errors(participant):
            raise ParticipationError(errors)
        participation = self.get_participation_for(participant)
        participation.state = AbstractParticipation.States.USER_DECLINED
        participation.save()
        return participation

    def get_configuration_fields(self):
        return {
            "minimum_age": {
                "formfield": forms.IntegerField(required=False),
                "default": 16,
                "publish_with_label": _("Minimum age"),
            },
            "signup_until": {
                "formfield": forms.SplitDateTimeField(
                    required=False, widget=CustomSplitDateTimeWidget
                ),
                "default": None,
                "publish_with_label": _("Signup until"),
                "format": functools.partial(formats.date_format, format="SHORT_DATETIME_FORMAT"),
            },
            "user_can_decline_confirmed": {
                "formfield": forms.BooleanField(
                    label=_("Confirmed users can decline by themselves"),
                    required=False,
                    help_text=_("only if the signup timeframe has not ended"),
                ),
                "default": False,
                "publish_with_label": _("Can decline after confirmation"),
                "format": yesno,
            },
        }

    def get_signup_info(self):
        """
        Return key/value pairs about the configuration to show in the shift info box.
        """
        fields = self.get_configuration_fields()
        return {
            label: field.get("format", str)(value)
            for key, field in fields.items()
            if (label := field.get("publish_with_label", False))
            and (value := getattr(self.configuration, key))
        }

    def render_shift_state(self, request):
        """
        Render html that will be shown in the shift info box.
        Use it to inform about the current state of the shift and participations.
        """
        return ""

    def get_configuration_form(self, *args, **kwargs):
        if self.shift is not None:
            kwargs.setdefault("initial", self.configuration.__dict__)
        form = ConfigurationForm(*args, **kwargs)
        for name, config in self.get_configuration_fields().items():
            form.fields[name] = config["formfield"]
        return form

    def render_configuration_form(self, form=None, *args, **kwargs):
        form = form or self.get_configuration_form(*args, **kwargs)
        template = Template(
            template_string="{% load bootstrap4 %}{% bootstrap_form form %}"
        ).render(Context({"form": form}))
        return template


class BaseSignupView(View):
    shift: Shift = ...
    method: BaseSignupMethod = ...

    def dispatch(self, request, *args, **kwargs):
        if (choice := request.POST.get("signup_choice")) is not None:
            if choice == "sign_up":
                return self.signup_pressed(request, *args, **kwargs)
            elif choice == "decline":
                return self.decline_pressed(request, *args, **kwargs)
            else:
                raise ValueError(
                    _("'{choice}' is not a valid signup action.").format(choice=choice)
                )
        return super().dispatch(request, *args, **kwargs)

    def signup_pressed(self, request, *args, **kwargs):
        try:
            with transaction.atomic():
                self.method.perform_signup(request.user.as_participant())
                messages.success(
                    request,
                    self.method.signup_success_message.format(shift=self.shift),
                )
        except ParticipationError as errors:
            for error in errors:
                messages.error(request, self.method.signup_error_message.format(error=error))
        finally:
            return redirect(self.shift.event.get_absolute_url())

    def decline_pressed(self, request, *args, **kwargs):
        try:
            with transaction.atomic():
                self.method.perform_decline(request.user.as_participant())
                messages.info(request, self.method.decline_success_message.format(shift=self.shift))
        except ParticipationError as errors:
            for error in errors:
                messages.error(request, self.method.decline_error_message.format(error=error))
        finally:
            return redirect(self.shift.event.get_absolute_url())



# Generated by Django 3.0.9 on 2020-09-06 19:11

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("event_management", "0007_auto_20200902_0016"),
    ]

    operations = [
        migrations.AddField(
            model_name="event",
            name="mail_updates",
            field=models.BooleanField(default=True, verbose_name="send updates via mail"),
        ),
    ]



from django.contrib.auth.password_validation import MinimumLengthValidator
from django.core.mail import EmailMultiAlternatives
from django.template.loader import render_to_string
from django.utils.translation import gettext as _

from ephios.settings import SITE_URL


class CustomMinimumLengthValidator(MinimumLengthValidator):
    def password_changed(self, password, user):
        if user is not None:
            text_content = _(
                "Your password for {site} has been changed. If you didn't request this change, contact an administrator immediately."
            ).format(site=SITE_URL)
            html_content = render_to_string("email_base.html", {"message_text": text_content})
            message = EmailMultiAlternatives(
                to=[user.email],
                subject=_("Your password has been changed"),
                body=text_content,
            )
            message.attach_alternative(html_content, "text/html")
            message.send()



# Generated by Django 3.0.6 on 2020-08-18 21:05

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("auth", "0011_update_proxy_permissions"),
    ]

    operations = [
        migrations.CreateModel(
            name="UserProfile",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("password", models.CharField(max_length=128, verbose_name="password")),
                (
                    "last_login",
                    models.DateTimeField(blank=True, null=True, verbose_name="last login"),
                ),
                (
                    "is_superuser",
                    models.BooleanField(
                        default=False,
                        help_text="Designates that this user has all permissions without explicitly assigning them.",
                        verbose_name="superuser status",
                    ),
                ),
                (
                    "email",
                    models.EmailField(max_length=254, unique=True, verbose_name="Email address"),
                ),
                ("is_active", models.BooleanField(default=True)),
                ("is_staff", models.BooleanField(default=False)),
                ("first_name", models.CharField(max_length=254, verbose_name="First name")),
                ("last_name", models.CharField(max_length=254, verbose_name="Last name")),
                ("date_of_birth", models.DateField()),
                ("phone", models.CharField(max_length=254, null=True, blank=True)),
                (
                    "groups",
                    models.ManyToManyField(
                        blank=True,
                        help_text="The groups this user belongs to. A user will get all permissions granted to each of their groups.",
                        related_name="user_set",
                        related_query_name="user",
                        to="auth.Group",
                        verbose_name="groups",
                    ),
                ),
                (
                    "user_permissions",
                    models.ManyToManyField(
                        blank=True,
                        help_text="Specific permissions for this user.",
                        related_name="user_set",
                        related_query_name="user",
                        to="auth.Permission",
                        verbose_name="user permissions",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="Qualification",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("title", models.CharField(max_length=254)),
            ],
        ),
        migrations.CreateModel(
            name="QualificationTrack",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("title", models.CharField(max_length=254)),
            ],
        ),
        migrations.CreateModel(
            name="QualificationGrant",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("expiration_date", models.DateField(blank=True, null=True)),
                (
                    "qualification",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="user_management.Qualification",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL
                    ),
                ),
            ],
        ),
        migrations.AddField(
            model_name="qualification",
            name="track",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE, to="user_management.QualificationTrack"
            ),
        ),
    ]



# Generated by Django 3.0.6 on 2020-08-29 21:48

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("event_management", "0004_auto_20200829_1535"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="event",
            options={"verbose_name": "event", "verbose_name_plural": "events"},
        ),
        migrations.AlterModelOptions(
            name="eventtype",
            options={"verbose_name": "event type", "verbose_name_plural": "event types"},
        ),
        migrations.AlterModelOptions(
            name="shift",
            options={"verbose_name": "shift", "verbose_name_plural": "shifts"},
        ),
        migrations.AlterField(
            model_name="abstractparticipation",
            name="shift",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to="event_management.Shift",
                verbose_name="shift",
            ),
        ),
        migrations.AlterField(
            model_name="abstractparticipation",
            name="state",
            field=models.IntegerField(
                choices=[
                    (0, "requested"),
                    (1, "confirmed"),
                    (2, "declined by user"),
                    (3, "rejected by responsible"),
                ],
                default=0,
                verbose_name="state",
            ),
        ),
        migrations.AlterField(
            model_name="event",
            name="description",
            field=models.TextField(blank=True, null=True, verbose_name="description"),
        ),
        migrations.AlterField(
            model_name="event",
            name="location",
            field=models.CharField(max_length=254, verbose_name="location"),
        ),
        migrations.AlterField(
            model_name="event",
            name="title",
            field=models.CharField(max_length=254, verbose_name="title"),
        ),
        migrations.AlterField(
            model_name="event",
            name="type",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to="event_management.EventType",
                verbose_name="event type",
            ),
        ),
        migrations.AlterField(
            model_name="eventtype",
            name="can_grant_qualification",
            field=models.BooleanField(verbose_name="can grant qualification"),
        ),
        migrations.AlterField(
            model_name="eventtype",
            name="title",
            field=models.CharField(max_length=254, verbose_name="title"),
        ),
        migrations.AlterField(
            model_name="shift",
            name="end_time",
            field=models.DateTimeField(verbose_name="end time"),
        ),
        migrations.AlterField(
            model_name="shift",
            name="event",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="shifts",
                to="event_management.Event",
                verbose_name="shifts",
            ),
        ),
        migrations.AlterField(
            model_name="shift",
            name="meeting_time",
            field=models.DateTimeField(verbose_name="meeting time"),
        ),
        migrations.AlterField(
            model_name="shift",
            name="signup_method_slug",
            field=models.SlugField(verbose_name="signup method"),
        ),
        migrations.AlterField(
            model_name="shift",
            name="start_time",
            field=models.DateTimeField(verbose_name="start time"),
        ),
    ]



# Generated by Django 3.0.6 on 2020-08-29 21:48

import secrets

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("user_management", "0005_qualification_abbreviation"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="qualification",
            options={"verbose_name": "qualification", "verbose_name_plural": "qualifications"},
        ),
        migrations.AlterModelOptions(
            name="qualificationcategory",
            options={
                "verbose_name": "qualification track",
                "verbose_name_plural": "qualification tracks",
            },
        ),
        migrations.AlterModelOptions(
            name="userprofile",
            options={"verbose_name": "user profile", "verbose_name_plural": "user profiles"},
        ),
        migrations.AlterField(
            model_name="qualification",
            name="category",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="qualifications",
                to="user_management.QualificationCategory",
                verbose_name="category",
            ),
        ),
        migrations.AlterField(
            model_name="qualification",
            name="included_qualifications",
            field=models.ManyToManyField(
                related_name="included_by", to="user_management.Qualification"
            ),
        ),
        migrations.AlterField(
            model_name="qualification",
            name="title",
            field=models.CharField(max_length=254, verbose_name="title"),
        ),
        migrations.AlterField(
            model_name="qualificationcategory",
            name="title",
            field=models.CharField(max_length=254, verbose_name="title"),
        ),
        migrations.AlterField(
            model_name="qualificationgrant",
            name="expires",
            field=models.DateTimeField(blank=True, null=True, verbose_name="expiration date"),
        ),
        migrations.AlterField(
            model_name="qualificationgrant",
            name="qualification",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to="user_management.Qualification",
                verbose_name="qualification",
            ),
        ),
        migrations.AlterField(
            model_name="qualificationgrant",
            name="user",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to=settings.AUTH_USER_MODEL,
                verbose_name="user profile",
            ),
        ),
        migrations.AlterField(
            model_name="userprofile",
            name="calendar_token",
            field=models.CharField(
                default=secrets.token_urlsafe, max_length=254, verbose_name="calendar token"
            ),
        ),
        migrations.AlterField(
            model_name="userprofile",
            name="date_of_birth",
            field=models.DateField(verbose_name="date of birth"),
        ),
        migrations.AlterField(
            model_name="userprofile",
            name="email",
            field=models.EmailField(max_length=254, unique=True, verbose_name="email address"),
        ),
        migrations.AlterField(
            model_name="userprofile",
            name="first_name",
            field=models.CharField(max_length=254, verbose_name="first name"),
        ),
        migrations.AlterField(
            model_name="userprofile",
            name="last_name",
            field=models.CharField(max_length=254, verbose_name="last name"),
        ),
        migrations.AlterField(
            model_name="userprofile",
            name="phone",
            field=models.CharField(max_length=254, null=True, verbose_name="phone number"),
        ),
    ]



from django.db.models.signals import post_save
from django.dispatch import receiver

from ephios.event_management import mail
from ephios.event_management.models import LocalParticipation


@receiver(
    post_save,
    sender=LocalParticipation,
    dispatch_uid="ephios.event_management.signals.send_participation_state_changed_mail",
)
def send_participation_state_changed_mail(sender, instance, **kwargs):
    mail.participation_state_changed(instance)



import json

from django.core.serializers.json import DjangoJSONEncoder
from django.db.models import Model, QuerySet
from django.utils import dateparse


class CustomJSONEncoder(DjangoJSONEncoder):
    def default(self, obj):
        if isinstance(obj, QuerySet):
            return list(obj)
        elif isinstance(obj, Model):
            return obj.pk
        else:
            return super().default(obj)


class CustomJSONDecoder(json.JSONDecoder):
    def __init__(self, *args, **kargs):
        super().__init__(*args, object_hook=self.custom_hook, **kargs)

    def custom_hook(self, d):
        for k, v in d.items():
            if isinstance(v, str):
                if (dt := dateparse.parse_datetime(v)) is not None:
                    d[k] = dt
                if (dt := dateparse.parse_date(v)) is not None:
                    d[k] = dt
        return d



"""
Django settings for ephios project.

Generated by 'django-admin startproject' using Django 3.0.6.

For more information on this file, see
https://docs.djangoproject.com/en/3.0/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/3.0/ref/settings/
"""

import os
from email.utils import getaddresses

import environ

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

env = environ.Env()
# for syntax see https://django-environ.readthedocs.io/en/latest/
environ.Env.read_env(env_file=os.path.join(BASE_DIR, ".env"))

SECRET_KEY = env.str("SECRET_KEY")
DEBUG = env.bool("DEBUG")
ALLOWED_HOSTS = env.list("ALLOWED_HOSTS")
SITE_URL = env.str("SITE_URL")

if not DEBUG:
    SESSION_COOKIE_SECURE = True
    CSRF_COOKIE_SECURE = True
    X_FRAME_OPTIONS = "DENY"
    SECURE_CONTENT_TYPE_NOSNIFF = True
    SECURE_BROWSER_XSS_FILTER = True
    SECURE_SSL_REDIRECT = True
    SECURE_HSTS_SECONDS = 3600
    SECURE_HSTS_INCLUDE_SUBDOMAINS = True
    SECURE_REFERRER_POLICY = "same-origin"

# Application definition

INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "polymorphic",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "bootstrap4",
    "guardian",
    "django_select2",
    "jquery",
    "djangoformsetjs",
    "ephios.user_management",
    "ephios.event_management",
    "ephios.extra",
    "ephios.plugins.basesignup",
    "ephios.plugins.pages",
]

MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
    "csp.middleware.CSPMiddleware",
]

ROOT_URLCONF = "ephios.urls"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [os.path.join(BASE_DIR, "ephios/templates")],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
                "ephios.extra.context.ephios_base_context",
            ],
        },
    },
]

LOCALE_PATHS = (os.path.join(BASE_DIR, "ephios/locale"),)

WSGI_APPLICATION = "ephios.wsgi.application"

# Database
# https://docs.djangoproject.com/en/3.0/ref/settings/#databases

DATABASES = {"default": env.db_url()}

# Password validation
# https://docs.djangoproject.com/en/3.0/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator",
    },
    {
        "NAME": "ephios.user_management.utils.CustomMinimumLengthValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.CommonPasswordValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.NumericPasswordValidator",
    },
]

AUTHENTICATION_BACKENDS = (
    "django.contrib.auth.backends.ModelBackend",
    "guardian.backends.ObjectPermissionBackend",
)

AUTH_USER_MODEL = "user_management.UserProfile"
LOGIN_REDIRECT_URL = "/"

# Internationalization
# https://docs.djangoproject.com/en/3.0/topics/i18n/

LANGUAGE_CODE = "de-de"

TIME_ZONE = "Europe/Berlin"

USE_I18N = True

USE_L10N = True

USE_TZ = True

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/3.0/howto/static-files/

STATIC_URL = env.str("STATIC_URL")
STATIC_ROOT = env.str("STATIC_ROOT")
STATICFILES_DIRS = (os.path.join(BASE_DIR, "ephios/static"),)

# mail configuration
EMAIL_CONFIG = env.email_url("EMAIL_URL")
vars().update(EMAIL_CONFIG)
DEFAULT_FROM_EMAIL = env.str("DEFAULT_FROM_EMAIL")
SERVER_EMAIL = env.str("SERVER_EMAIL")
ADMINS = getaddresses([env("ADMINS")])

# Guardian configuration
ANONYMOUS_USER_NAME = None
GUARDIAN_MONKEY_PATCH = False

# django-select2
# Prevent django-select from loading the select2 resources as we want to serve them locally
SELECT2_JS = ""
SELECT2_CSS = ""
SELECT2_I18N_PATH = ""

# django-debug-toolbar
if DEBUG:
    INSTALLED_APPS.append("django_extensions")
    INSTALLED_APPS.append("debug_toolbar")
    MIDDLEWARE.insert(0, "debug_toolbar.middleware.DebugToolbarMiddleware")
    INTERNAL_IPS = env.str("INTERNAL_IPS")

# django-csp
# Bootstrap requires embedded SVG files loaded via a data URI. This is not ideal, but will only be fixed in
# bootstrap v5 or v6. See https://github.com/twbs/bootstrap/issues/25394 for details on the problem and
# https://security.stackexchange.com/a/167244 on why allowing data: is considered okay
CSP_IMG_SRC = ("'self'", "data:")



# Generated by Django 3.1.1 on 2020-09-25 14:54

import secrets
import uuid

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    replaces = [
        ("user_management", "0003_userprofile_calendar_token"),
        ("user_management", "0004_auto_20200829_1535"),
        ("user_management", "0005_qualification_abbreviation"),
        ("user_management", "0006_auto_20200829_2348"),
        ("user_management", "0007_auto_20200921_2155"),
        ("user_management", "0008_auto_20200925_1640"),
    ]

    dependencies = [
        ("user_management", "0002_initial_permissions"),
    ]

    operations = [
        migrations.AddField(
            model_name="userprofile",
            name="calendar_token",
            field=models.CharField(default=secrets.token_urlsafe, max_length=254),
        ),
        migrations.CreateModel(
            name="QualificationCategory",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("uuid", models.UUIDField(default=uuid.uuid4, unique=True)),
                ("title", models.CharField(max_length=254, verbose_name="title")),
            ],
            options={
                "verbose_name": "qualification track",
                "verbose_name_plural": "qualification tracks",
            },
        ),
        migrations.RemoveField(
            model_name="qualification",
            name="track",
        ),
        migrations.RemoveField(
            model_name="qualificationgrant",
            name="expiration_date",
        ),
        migrations.AddField(
            model_name="qualification",
            name="included_qualifications",
            field=models.ManyToManyField(
                related_name="included_in_set", to="user_management.Qualification"
            ),
        ),
        migrations.AddField(
            model_name="qualification",
            name="uuid",
            field=models.UUIDField(default=None, unique=True),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="qualificationgrant",
            name="expires",
            field=models.DateTimeField(blank=True, null=True, verbose_name="expiration date"),
        ),
        migrations.AlterField(
            model_name="qualificationgrant",
            name="qualification",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="grants",
                to="user_management.qualification",
            ),
        ),
        migrations.AlterField(
            model_name="qualificationgrant",
            name="user",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="qualification_grants",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
        migrations.DeleteModel(
            name="QualificationTrack",
        ),
        migrations.AddField(
            model_name="qualification",
            name="category",
            field=models.ForeignKey(
                default=None,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="qualifications",
                to="user_management.qualificationcategory",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="qualification",
            name="abbreviation",
            field=models.CharField(default="", max_length=254),
            preserve_default=False,
        ),
        migrations.AlterModelOptions(
            name="qualification",
            options={"verbose_name": "qualification", "verbose_name_plural": "qualifications"},
        ),
        migrations.AlterModelOptions(
            name="userprofile",
            options={"verbose_name": "user profile", "verbose_name_plural": "user profiles"},
        ),
        migrations.AlterField(
            model_name="qualification",
            name="category",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="qualifications",
                to="user_management.qualificationcategory",
                verbose_name="category",
            ),
        ),
        migrations.AlterField(
            model_name="qualification",
            name="included_qualifications",
            field=models.ManyToManyField(
                related_name="included_by", to="user_management.Qualification"
            ),
        ),
        migrations.AlterField(
            model_name="qualification",
            name="title",
            field=models.CharField(max_length=254, verbose_name="title"),
        ),
        migrations.AlterField(
            model_name="qualificationgrant",
            name="qualification",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to="user_management.qualification",
                verbose_name="qualification",
            ),
        ),
        migrations.AlterField(
            model_name="qualificationgrant",
            name="user",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to=settings.AUTH_USER_MODEL,
                verbose_name="user profile",
            ),
        ),
        migrations.AlterField(
            model_name="userprofile",
            name="calendar_token",
            field=models.CharField(
                default=secrets.token_urlsafe, max_length=254, verbose_name="calendar token"
            ),
        ),
        migrations.AlterField(
            model_name="userprofile",
            name="date_of_birth",
            field=models.DateField(verbose_name="date of birth"),
        ),
        migrations.AlterField(
            model_name="userprofile",
            name="email",
            field=models.EmailField(max_length=254, unique=True, verbose_name="email address"),
        ),
        migrations.AlterField(
            model_name="userprofile",
            name="first_name",
            field=models.CharField(max_length=254, verbose_name="first name"),
        ),
        migrations.AlterField(
            model_name="userprofile",
            name="last_name",
            field=models.CharField(max_length=254, verbose_name="last name"),
        ),
        migrations.AlterField(
            model_name="userprofile",
            name="phone",
            field=models.CharField(max_length=254, null=True, verbose_name="phone number"),
        ),
        migrations.AlterField(
            model_name="qualification",
            name="uuid",
            field=models.UUIDField(default=uuid.uuid4, unique=True),
        ),
        migrations.AlterField(
            model_name="qualificationgrant",
            name="qualification",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="grants",
                to="user_management.qualification",
                verbose_name="qualification",
            ),
        ),
        migrations.AlterField(
            model_name="userprofile",
            name="phone",
            field=models.CharField(
                blank=True, default="", max_length=254, verbose_name="phone number"
            ),
            preserve_default=False,
        ),
    ]



from django import forms
from django.shortcuts import redirect
from django.template.loader import get_template
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from django.views.generic import TemplateView
from django.views.generic.detail import SingleObjectMixin

from ephios.event_management.models import AbstractParticipation, Shift
from ephios.extra.permissions import CustomPermissionRequiredMixin
from ephios.plugins.basesignup.signup.instant import SimpleQualificationsRequiredSignupMethod

DispositionParticipationFormset = forms.modelformset_factory(
    model=AbstractParticipation,
    fields=["state"],
    extra=0,
    can_order=False,
    can_delete=False,
    widgets={
        "state": forms.HiddenInput(attrs={"class": "state-input"}),
    },
)


class RequestConfirmDispositionView(CustomPermissionRequiredMixin, SingleObjectMixin, TemplateView):
    model = Shift
    permission_required = "event_management.change_event"
    template_name = "basesignup/requestconfirm_signup/disposition.html"

    def get_permission_object(self):
        self.object: Shift = self.get_object()
        return self.object.event

    def get_formset(self):
        return DispositionParticipationFormset(
            self.request.POST or None, queryset=self.object.participations
        )

    def get(self, request, *args, **kwargs):
        self.object: Shift = self.get_object()
        return super().get(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        self.object: Shift = self.get_object()
        formset = self.get_formset()
        if formset.is_valid():
            formset.save()
            return redirect(self.object.event.get_absolute_url())
        return self.get(request, *args, **kwargs, formset=formset)

    def get_context_data(self, **kwargs):
        kwargs.setdefault("formset", self.get_formset())
        kwargs.setdefault("states", AbstractParticipation.States)
        return super().get_context_data(**kwargs)


class RequestConfirmSignupMethod(SimpleQualificationsRequiredSignupMethod):
    slug = "request_confirm"
    verbose_name = _("Request and confirm")
    description = _(
        """This method lets people request participation. Responsibles can then confirm the participation."""
    )
    registration_button_text = _("Request")
    signup_success_message = _("You have successfully requested a participation for {shift}.")
    signup_error_message = _("Requesting a participation failed: {error}")

    def render_shift_state(self, request):
        participations = self.shift.participations.filter(
            state__in={
                AbstractParticipation.States.REQUESTED,
                AbstractParticipation.States.CONFIRMED,
            }
        )
        return get_template("basesignup/requestconfirm_signup/fragment_state.html").render(
            {
                "shift": self.shift,
                "requested_participants": (
                    p.participant
                    for p in participations.filter(state=AbstractParticipation.States.REQUESTED)
                ),
                "confirmed_participants": (
                    p.participant
                    for p in participations.filter(state=AbstractParticipation.States.CONFIRMED)
                ),
                "disposition_url": (
                    reverse(
                        "basesignup:shift_disposition_requestconfirm", kwargs=dict(pk=self.shift.pk)
                    )
                    if request.user.has_perm("event_management.change_event", obj=self.shift.event)
                    else None
                ),
            }
        )

    def perform_signup(self, participant, **kwargs):
        participation = super().perform_signup(participant, **kwargs)
        participation.state = AbstractParticipation.States.REQUESTED
        participation.save()
        return participation



# Generated by Django 3.0.6 on 2020-08-25 11:51

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("event_management", "0002_event_active"),
    ]

    operations = [
        migrations.AlterField(
            model_name="shift",
            name="event",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="shifts",
                to="event_management.Event",
            ),
        ),
        migrations.AlterField(
            model_name="shift",
            name="signup_method_slug",
            field=models.SlugField(verbose_name="Signup method"),
        ),
    ]



import importlib
import subprocess

from django.templatetags.static import static
from django.utils.translation import get_language

from ephios.extra.signals import footer_link

try:
    EPHIOS_VERSION = (
        subprocess.check_output(["git", "rev-parse", "--short", "HEAD"]).decode().strip()
    )
except (subprocess.CalledProcessError, FileNotFoundError):
    # suggested in https://github.com/python-poetry/poetry/issues/273
    EPHIOS_VERSION = "v" + importlib.metadata.version("ephios")


def ephios_base_context(request):
    footer = {}
    for receiver, result in footer_link.send(None, request=request):
        for label, url in result.items():
            footer[label] = url

    datatables_translation_url = None
    if get_language() == "de-de":
        datatables_translation_url = static("datatables/german.json")

    return {
        "footer": footer,
        "datatables_translation_url": datatables_translation_url,
        "ephios_version": EPHIOS_VERSION,
    }



import io

import pytz
from django.http import FileResponse
from django.utils import formats
from django.utils.translation import gettext as _
from django.views import View
from django.views.generic.detail import SingleObjectMixin
from reportlab.lib.pagesizes import A4, A5
from reportlab.lib.styles import getSampleStyleSheet
from reportlab.lib.units import cm
from reportlab.platypus import Paragraph, SimpleDocTemplate, Spacer, Table

from ephios import settings
from ephios.event_management.models import Event
from ephios.extra.permissions import CustomPermissionRequiredMixin


class BasePDFExporter:
    def __init__(self, title, style=getSampleStyleSheet(), pagesize=A4):
        self.title = title
        self.style = style
        self.pagesize = pagesize

    def get_pdf(self):
        buffer = io.BytesIO()
        story = self.get_story()
        p = SimpleDocTemplate(
            buffer,
            pagesize=self.pagesize,
            title=self.title,
            leftMargin=1 * cm,
            rightMargin=1 * cm,
            topMargin=1 * cm,
            bottomMargin=1 * cm,
        )
        p.build(story)
        buffer.seek(0)
        return FileResponse(buffer, as_attachment=True, filename=f"{self.title}.pdf")

    def get_story(self):
        return NotImplemented


class SingleShiftEventExporter(BasePDFExporter):
    def __init__(self, event, **kwargs):
        self.event = event
        super().__init__(title=event.title, pagesize=A5)

    def get_story(self):
        story = [
            Paragraph(f"{self.event.type}: {self.event.title}", self.style["Heading1"]),
            Spacer(height=0.5 * cm, width=15 * cm),
        ]

        shift = self.event.shifts.first()
        tz = pytz.timezone(settings.TIME_ZONE)
        start_time = shift.start_time.astimezone(tz)
        data = (
            [
                [_("Location"), self.event.location],
                [
                    _("Date"),
                    f"{formats.date_format(start_time, 'l')}, {formats.date_format(start_time, 'SHORT_DATE_FORMAT')}",
                ],
                [
                    _("Time"),
                    f"{formats.time_format(start_time)} - {formats.time_format(shift.end_time.astimezone(tz))}",
                ],
                [_("Meeting time"), formats.time_format(shift.meeting_time.astimezone(tz))],
            ]
            + [
                [Paragraph(key), Paragraph(value)]
                for key, value in shift.signup_method.get_signup_info().items()
            ]
            + [[_("Description"), Paragraph(self.event.description)]]
        )
        table = Table(data, colWidths=[5.7 * cm, 7 * cm])
        table.setStyle([("VALIGN", (0, -1), (-1, -1), "TOP")])
        story.append(table)
        story.append(Spacer(height=0.5 * cm, width=15 * cm))

        data = [
            [f"{participant.first_name} {participant.last_name}"]
            for participant in shift.get_participants()
        ]
        if data:
            story.append(Paragraph(_("Participants"), self.style["Heading2"]))
            story.append(Table(data, colWidths=[12.7 * cm]))

        return story


class MultipleShiftEventExporter(BasePDFExporter):
    def __init__(self, event):
        self.event = event
        super().__init__(title=event.title)

    def get_story(self):
        story = [
            Paragraph(f"{self.event.type}: {self.event.title}", self.style["Heading1"]),
            Spacer(height=0.5 * cm, width=19 * cm),
        ]

        tz = pytz.timezone(settings.TIME_ZONE)
        start_time = self.event.get_start_time().astimezone(tz)
        end_time = self.event.get_end_time().astimezone(tz)
        end_date = (
            f"- {formats.date_format(end_time, 'l')}, {formats.date_format(end_time, 'SHORT_DATE_FORMAT')}"
            if end_time.date() > start_time.date()
            else ""
        )
        event_date = f"{formats.date_format(start_time, 'l')}, {formats.date_format(start_time, 'SHORT_DATE_FORMAT')} {end_date}"
        data = [
            [_("Location"), self.event.location],
            [_("Date"), event_date],
            [_("Description"), Paragraph(self.event.description)],
        ]
        table = Table(data, colWidths=[6 * cm, 13 * cm])
        table.setStyle([("VALIGN", (0, -1), (-1, -1), "TOP")])
        story.append(table)

        for shift in self.event.shifts.all():
            story.append(Spacer(height=1 * cm, width=19 * cm))
            story.append(Paragraph(shift.get_start_end_time_display(), self.style["Heading2"]))
            data = [
                [_("Meeting time"), formats.time_format(shift.meeting_time.astimezone(tz))],
            ] + [
                [Paragraph(key), Paragraph(value)]
                for key, value in shift.signup_method.get_signup_info().items()
            ]
            story.append(Table(data, colWidths=[6 * cm, 13 * cm]))

            data = [
                [f"{participant.first_name} {participant.last_name}"]
                for participant in shift.get_participants()
            ]
            if data:
                story.append(Paragraph(_("Participants"), self.style["Heading3"]))
                story.append(Table(data, colWidths=[19 * cm]))

        return story


class EventDetailPDFView(CustomPermissionRequiredMixin, SingleObjectMixin, View):
    permission_required = "event_management.view_event"
    model = Event

    def get(self, request, *args, **kwargs):
        event = self.get_object()
        if event.shifts.count() > 1:
            return MultipleShiftEventExporter(event=event).get_pdf()
        else:
            return SingleShiftEventExporter(event=event).get_pdf()



from django.urls import path

from ephios.event_management import pdf, views
from ephios.event_management.ical import EventFeed, user_event_feed_view
from ephios.event_management.views import ShiftConfigurationFormView

app_name = "event_management"
urlpatterns = [
    path("", views.HomeView.as_view(), name="index"),
    path("events/", views.EventListView.as_view(), name="event_list"),
    path("events/<int:pk>/edit/", views.EventUpdateView.as_view(), name="event_edit"),
    path("events/<int:pk>/delete/", views.EventDeleteView.as_view(), name="event_delete"),
    path("events/<int:pk>/", views.EventDetailView.as_view(), name="event_detail"),
    path("events/<int:pk>/createshift/", views.ShiftCreateView.as_view(), name="event_createshift"),
    path("events/<int:pk>/activate/", views.EventActivateView.as_view(), name="event_activate"),
    path("events/<int:pk>/pdf/", pdf.EventDetailPDFView.as_view(), name="event_detail_pdf"),
    path("events/create/", views.EventCreateView.as_view(), name="event_create"),
    path("events/past/", views.EventArchiveView.as_view(), name="event_list_past"),
    path(
        "shifts/<int:pk>/signup-action/",
        views.ShiftSignupView.as_view(),
        name="shift_action",
    ),
    path(
        "shifts/<int:pk>/edit/",
        views.ShiftUpdateView.as_view(),
        name="shift_edit",
    ),
    path(
        "shifts/<int:pk>/delete/",
        views.ShiftDeleteView.as_view(),
        name="shift_delete",
    ),
    path(
        "signup_methods/<slug:slug>/configuration_form/",
        ShiftConfigurationFormView.as_view(),
        name="signupmethod_configurationform",
    ),
    path("calendar/", EventFeed(), name="event_feed"),
    path("calendar/<str:calendar_token>/", user_event_feed_view, name="user_event_feed"),
]



from django.forms import DateInput, MultiWidget, SplitDateTimeWidget, TimeInput
from django.forms.utils import to_current_timezone


class CustomDateInput(DateInput):
    template_name = "event_management/fragments/custom_date_input.html"


class CustomTimeInput(TimeInput):
    template_name = "event_management/fragments/custom_time_input.html"


class CustomSplitDateTimeInput(SplitDateTimeWidget):
    pass


class CustomSplitDateTimeWidget(MultiWidget):
    """
    A widget that splits datetime input into two <input type="text"> boxes.
    """

    supports_microseconds = False
    template_name = "event_management/fragments/custom_split_date_time_widget.html"

    def __init__(self, *args, **kwargs):
        widgets = (
            CustomDateInput(format="%Y-%m-%d"),
            CustomTimeInput(format="%H:%M"),
        )
        super().__init__(widgets, *args, **kwargs)

    def decompress(self, value):
        if value:
            value = to_current_timezone(value)
            return [value.date(), value.time()]
        return [None, None]



from django.core.exceptions import PermissionDenied
from django.views.generic import DetailView

from ephios.plugins.pages.models import Page


class PageView(DetailView):
    model = Page
    template_name = "pages/view_page.html"

    def setup(self, request, *args, **kwargs):
        super().setup(request, *args, **kwargs)
        self.object = self.get_object()

    def dispatch(self, request, *args, **kwargs):
        if not request.user.is_authenticated and not self.object.publicly_visible:
            raise PermissionDenied
        return super().dispatch(request, *args, **kwargs)



from django.core import mail
from django.core.mail import EmailMultiAlternatives
from django.template.loader import render_to_string
from django.utils.translation import gettext as _
from guardian.shortcuts import get_users_with_perms

from ephios.event_management.models import AbstractParticipation, LocalParticipation
from ephios.extra.permissions import get_groups_with_perms
from ephios.settings import SITE_URL
from ephios.user_management.models import UserProfile


def new_event(event):
    messages = []
    users = UserProfile.objects.filter(
        groups__in=get_groups_with_perms(event, only_with_perms_in=["view_event"]), is_active=True
    ).distinct()
    responsible_persons = get_users_with_perms(
        event, only_with_perms_in=["change_event"]
    ).distinct()
    responsible_persons_mails = list(responsible_persons.values_list("email", flat=True))

    subject = _("New {type}: {title}").format(type=event.type, title=event.title)
    text_content = _(
        "A new {type} ({title}) has been added. \n You can view it here: {link}"
    ).format(type=event.type, title=event.title, link=event.get_absolute_url())
    html_content = render_to_string(
        "event_management/mails/new_event.html", {"event": event, "site_url": SITE_URL}
    )

    for user in users:
        message = EmailMultiAlternatives(
            to=[user.email], subject=subject, body=text_content, reply_to=responsible_persons_mails
        )
        message.attach_alternative(html_content, "text/html")
        messages.append(message)
    mail.get_connection().send_messages(messages)


def participation_state_changed(participation: AbstractParticipation):
    if participation.state != AbstractParticipation.States.USER_DECLINED:
        messages = []

        # send mail to the participant whose participation has been changed
        if participation.participant.email is not None:
            text_content = _(
                "The status for your participation for {shift} has changed. It is now {status}."
            ).format(shift=participation.shift, status=participation.get_state_display())
            html_content = render_to_string("email_base.html", {"message_text": text_content})
            message = EmailMultiAlternatives(
                to=[participation.participant.email],
                subject=_("Your participation state changed"),
                body=text_content,
            )
            message.attach_alternative(html_content, "text/html")
            messages.append(message)

        # send mail to responsible users
        responsible_persons = get_users_with_perms(
            participation.shift.event, only_with_perms_in=["change_event"]
        ).distinct()
        subject = _("Participation was changed for your event")
        text_content = _(
            "The participation of {participant} for {shift} was changed. The status is now {status}"
        ).format(
            participant=participation.participant,
            shift=participation.shift,
            status=participation.get_state_display(),
        )
        html_content = render_to_string("email_base.html", {"message_text": text_content})
        for user in responsible_persons:
            message = EmailMultiAlternatives(to=[user.email], subject=subject, body=text_content)
            message.attach_alternative(html_content, "text/html")
            messages.append(message)

        mail.get_connection().send_messages(messages)



from django.apps import AppConfig


class UserManagementConfig(AppConfig):
    name = "ephios.user_management"

    def ready(self):
        from ephios.user_management import signals


default_app_config = "ephios.user_management.UserManagementConfig"



# Generated by Django 3.1.1 on 2020-09-21 19:55

import uuid

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("user_management", "0006_auto_20200829_2348"),
    ]

    operations = [
        migrations.AlterField(
            model_name="qualification",
            name="uuid",
            field=models.UUIDField(default=uuid.uuid4, unique=True),
        ),
        migrations.AlterField(
            model_name="qualificationcategory",
            name="uuid",
            field=models.UUIDField(default=uuid.uuid4, unique=True),
        ),
    ]



from django.apps import AppConfig


class EventManagementConfig(AppConfig):
    name = "ephios.event_management"

    def ready(self):
        from ephios.event_management import signals


default_app_config = "ephios.event_management.EventManagementConfig"



from typing import TYPE_CHECKING

import pytz
from django.db import models, transaction
from django.db.models import (
    BooleanField,
    CharField,
    DateTimeField,
    ForeignKey,
    IntegerField,
    JSONField,
    Manager,
    Model,
    SlugField,
    TextField,
)
from django.utils import formats
from django.utils.functional import cached_property
from django.utils.translation import gettext_lazy as _
from polymorphic.models import PolymorphicModel

from ephios import settings
from ephios.user_management.models import UserProfile

if TYPE_CHECKING:
    from ephios.event_management.signup import AbstractParticipant


class ActiveManager(Manager):
    def get_queryset(self):
        return super().get_queryset().filter(active=True)


class EventType(Model):
    title = CharField(_("title"), max_length=254)
    can_grant_qualification = BooleanField(_("can grant qualification"))

    class Meta:
        verbose_name = _("event type")
        verbose_name_plural = _("event types")

    def __str__(self):
        return self.title


class EventSeries(Model):
    pass


class Event(Model):
    title = CharField(_("title"), max_length=254)
    description = TextField(_("description"), blank=True, null=True)
    location = CharField(_("location"), max_length=254)
    type = ForeignKey(EventType, on_delete=models.CASCADE, verbose_name=_("event type"))
    series = ForeignKey(EventSeries, on_delete=models.CASCADE, blank=True, null=True)
    active = BooleanField(default=False)
    mail_updates = BooleanField(_("send updates via mail"), default=True)

    objects = ActiveManager()
    all_objects = Manager()

    class Meta:
        verbose_name = _("event")
        verbose_name_plural = _("events")
        permissions = [("view_past_event", _("Can view past events"))]

    def get_start_time(self):
        if (first_shift := self.shifts.order_by("start_time").first()) is not None:
            return first_shift.start_time

    def get_end_time(self):
        if (last_shift := self.shifts.order_by("end_time").last()) is not None:
            return last_shift.end_time

    def __str__(self):
        return self.title

    def get_absolute_url(self):
        from django.urls import reverse

        return reverse("event_management:event_detail", args=[str(self.id)])

    def activate(self):
        from ephios.event_management import mail

        if not self.active:
            with transaction.atomic():
                self.active = True
                self.full_clean()
                self.save()
                if self.mail_updates:
                    mail.new_event(self)


class AbstractParticipation(PolymorphicModel):
    class States(models.IntegerChoices):
        REQUESTED = 0, _("requested")
        CONFIRMED = 1, _("confirmed")
        USER_DECLINED = 2, _("declined by user")
        RESPONSIBLE_REJECTED = 3, _("rejected by responsible")

    shift = ForeignKey("Shift", on_delete=models.CASCADE, verbose_name=_("shift"))
    state = IntegerField(_("state"), choices=States.choices, default=States.REQUESTED)

    @property
    def participant(self) -> "AbstractParticipant":
        raise NotImplementedError

    def __str__(self):
        try:
            return f"{self.participant} @ {self.shift}"
        except NotImplementedError:
            return super().__str__()


class Shift(Model):
    event = ForeignKey(
        Event, on_delete=models.CASCADE, related_name="shifts", verbose_name=_("shifts")
    )
    meeting_time = DateTimeField(_("meeting time"))
    start_time = DateTimeField(_("start time"))
    end_time = DateTimeField(_("end time"))
    signup_method_slug = SlugField(_("signup method"))
    signup_configuration = JSONField(default=dict)

    class Meta:
        verbose_name = _("shift")
        verbose_name_plural = _("shifts")

    @cached_property
    def signup_method(self):
        from ephios.event_management.signup import signup_method_from_slug

        return signup_method_from_slug(self.signup_method_slug, self)

    def get_start_end_time_display(self):
        tz = pytz.timezone(settings.TIME_ZONE)
        start_time = self.start_time.astimezone(tz)
        return (
            f"{formats.date_format(start_time, 'l')}, {formats.date_format(start_time, 'SHORT_DATE_FORMAT')}, "
            + f"{formats.time_format(start_time)} - {formats.time_format(self.end_time.astimezone(tz))}"
        )

    @property
    def participations(self):
        return AbstractParticipation.objects.filter(shift=self)

    def get_participants(self, with_state_in=frozenset({AbstractParticipation.States.CONFIRMED})):
        yield from (
            participation.participant
            for participation in self.participations.filter(state__in=with_state_in)
        )

    def __str__(self):
        return f"{self.event.title} ({self.get_start_end_time_display()})"


class LocalParticipation(AbstractParticipation):
    user = ForeignKey(UserProfile, on_delete=models.CASCADE)

    @property
    def participant(self):
        return self.user.as_participant()



# Generated by Django 3.0.6 on 2020-08-18 21:05
from django.db import migrations

# This does not use historical models, but the current version of Permission
# It might break if models change in the future.


def create_initial_permissions(apps, schema_editor):
    from django.contrib.auth.models import Group, Permission
    from django.contrib.contenttypes.models import ContentType

    group_content_type = ContentType.objects.get_for_model(Group)
    Permission.objects.create(
        name="Publish event for group",
        codename="publish_event_for_group",
        content_type=group_content_type,
    )


class Migration(migrations.Migration):
    dependencies = [
        ("user_management", "0001_initial"),
        ("guardian", "0001_initial"),
    ]

    operations = [
        migrations.RunPython(create_initial_permissions),
    ]



# Generated by Django 3.0.6 on 2020-08-27 14:04

import secrets

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("user_management", "0002_initial_permissions"),
    ]

    operations = [
        migrations.AddField(
            model_name="userprofile",
            name="calendar_token",
            field=models.CharField(default=secrets.token_urlsafe, max_length=254),
        ),
    ]



from django.contrib.auth.tokens import default_token_generator
from django.core.mail import EmailMultiAlternatives
from django.template.loader import render_to_string
from django.urls import reverse
from django.utils.encoding import force_bytes
from django.utils.http import urlsafe_base64_encode
from django.utils.translation import gettext as _

from ephios.settings import SITE_URL


def send_account_creation_info(userprofile):
    subject = _("Welcome to ephios!")
    uid = urlsafe_base64_encode(force_bytes(userprofile.id))
    token = default_token_generator.make_token(userprofile)
    reset_link = reverse("password_reset_confirm", kwargs={"uidb64": uid, "token": token})
    text_content = _(
        "You're receiving this email because a new account has been created for you at ephios.\n"
        "Please go to the following page and choose a password: {url}{reset_link}\n"
        "Your username is your email address: {email}\n"
    ).format(url=SITE_URL, reset_link=reset_link, email=userprofile.email)

    html_content = render_to_string(
        "user_management/new_account_email.html",
        {"uid": uid, "token": token, "site_url": SITE_URL, "email": userprofile.email},
    )
    message = EmailMultiAlternatives(to=[userprofile.email], subject=subject, body=text_content)
    message.attach_alternative(html_content, "text/html")
    message.send()


def send_account_update_info(userprofile):
    subject = _("ephios account updated")
    url = reverse("user_management:profile")
    text_content = _(
        "You're receiving this email because your account at ephios has been updated.\n"
        "You can see the changes in your profile: {site_url}{url}\n"
        "Your username is your email address: {email}\n"
    ).format(site_url=SITE_URL, url=url, email=userprofile.email)

    html_content = render_to_string(
        "user_management/account_updated_email.html",
        {"site_url": SITE_URL, "url": url, "email": userprofile.email},
    )
    message = EmailMultiAlternatives(to=[userprofile.email], subject=subject, body=text_content)
    message.attach_alternative(html_content, "text/html")
    message.send()



# Generated by Django 3.0.6 on 2020-08-18 21:04

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="Event",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("title", models.CharField(max_length=254)),
                ("description", models.TextField(blank=True, null=True)),
                ("location", models.CharField(max_length=254)),
            ],
        ),
        migrations.CreateModel(
            name="EventSeries",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="EventType",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("title", models.CharField(max_length=254)),
                ("can_grant_qualification", models.BooleanField()),
            ],
        ),
        migrations.CreateModel(
            name="Shift",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("meeting_time", models.DateTimeField()),
                ("start_time", models.DateTimeField()),
                ("end_time", models.DateTimeField()),
                ("signup_method_slug", models.SlugField()),
                ("signup_configuration", models.JSONField()),
                (
                    "event",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, to="event_management.Event"
                    ),
                ),
            ],
        ),
        migrations.AddField(
            model_name="event",
            name="series",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="event_management.EventSeries",
            ),
        ),
        migrations.AddField(
            model_name="event",
            name="type",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE, to="event_management.EventType"
            ),
        ),
        migrations.CreateModel(
            name="AbstractParticipation",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                (
                    "state",
                    models.IntegerField(
                        choices=[(0, "requested"), (1, "confirmed"), (2, "rejected")], default=0
                    ),
                ),
                (
                    "shift",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, to="event_management.Shift"
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="LocalParticipation",
            fields=[
                (
                    "abstractparticipation_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="event_management.AbstractParticipation",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL
                    ),
                ),
            ],
            bases=("event_management.abstractparticipation",),
        ),
    ]



import json
import uuid
from datetime import datetime

from django.contrib.auth.hashers import make_password
from django.core.management.base import BaseCommand
from django.core.serializers.json import DjangoJSONEncoder
from django.db import transaction
from django.utils.timezone import make_aware
from django.utils.translation import gettext as _
from guardian.shortcuts import assign_perm

from ephios.event_management.models import Event, EventType, Shift
from ephios.user_management.models import Qualification, QualificationCategory, UserProfile

AVAILABLE_DATASET_CLASSES = []


def register_dataset(dataset_type):
    AVAILABLE_DATASET_CLASSES.append(dataset_type)
    return dataset_type


class AbstractDataset:
    identifier = NotImplemented
    action = None

    def __init__(self, command):
        self.stdout = command.stdout

    def create_objects(self, *args, **options):
        self.stdout.write(self.description)

    @property
    def description(self):
        return "\n".join(klaas.action or str(klaas) for klaas in reversed(type(self).__mro__[:-2]))

    def add_arguments(self, parser):
        pass


@register_dataset
class AdminUserDataset(AbstractDataset):
    identifier = "admin_user"
    action = "Add a superuser admin@localhost with password 'admin'."

    def create_objects(self, *args, **options):
        super().create_objects(*args, **options)
        self.admin_user = UserProfile(
            email="admin@localhost",
            first_name="Admin",
            last_name="Localhost",
            date_of_birth=datetime(year=1970, month=1, day=1),
        )
        self.admin_user.is_staff = True
        self.admin_user.is_superuser = True
        self.admin_user.password = make_password("admin")
        self.admin_user.save()


@register_dataset
class DebugDataset(AdminUserDataset):
    identifier = "debug"
    action = "Create a planner and volunteers group and add the superuser and another user to it. Add an event and a few qualifications."

    def create_objects(self, *args, **options):
        super().create_objects(*args, **options)
        from django.contrib.auth.models import Group

        volunteers = Group.objects.create(name=_("Volunteers"))
        volunteers.user_set.add(self.admin_user)
        volunteers.save()

        planners = Group.objects.create(name=_("Planners"))
        planners.user_set.add(self.admin_user)
        planners.save()

        managers = Group.objects.create(name=_("Managers"))
        managers.user_set.add(self.admin_user)
        managers.save()

        assign_perm("publish_event_for_group", planners, volunteers)
        assign_perm("event_management.add_event", planners)
        assign_perm("event_management.delete_event", planners)
        assign_perm("event_management.view_past_event", planners)
        assign_perm("user_management.view_userprofile", managers)
        assign_perm("user_management.add_userprofile", managers)
        assign_perm("user_management.change_userprofile", managers)
        assign_perm("user_management.delete_userprofile", managers)
        assign_perm("auth.view_group", managers)
        assign_perm("auth.add_group", managers)
        assign_perm("auth.change_group", managers)
        assign_perm("auth.delete_group", managers)

        service_type = EventType.objects.create(title=_("Service"), can_grant_qualification=False)
        EventType.objects.create(title=_("Training"), can_grant_qualification=True)

        user = UserProfile(
            email="user@localhost",
            first_name="User",
            last_name="Localhost",
            date_of_birth=datetime(year=1970, month=1, day=1),
        )
        user.password = make_password("user")
        user.save()
        volunteers.user_set.add(user)

        event = Event.objects.create(
            title="Concert Medical Service",
            description="Your contact is Lisa Example. Her Phone number is 012345678910",
            type=service_type,
            location="Town Square Gardens",
            active=True,
        )

        assign_perm("event_management.view_event", volunteers, event)

        Shift.objects.create(
            event=event,
            meeting_time=make_aware(datetime(2023, 6, 30, 15, 30)),
            start_time=make_aware(datetime(2023, 6, 30, 16, 0)),
            end_time=make_aware(datetime(2023, 7, 1, 1, 0)),
            signup_method_slug="instant_confirmation",
            signup_configuration=json.dumps(
                dict(
                    minimum_age=18,
                    signup_until=make_aware(datetime(2023, 6, 29, 8, 0)),
                ),
                cls=DjangoJSONEncoder,
            ),
        )


@register_dataset
class QualificationDataset(AbstractDataset):
    identifier = "qualifications"
    action = "Create qualification objects for medical qualifications and driver licenses"

    def create_objects(self, *args, **options):
        medical_category = QualificationCategory.objects.create(
            title=_("Medical"),
            uuid=uuid.UUID("50380292-b9c9-4711-b70d-8e03e2784cfb"),
        )

        eh = Qualification.objects.create(
            category=medical_category,
            title="Ersthelfer",
            abbreviation="EH",
            uuid=uuid.UUID("8bdb343e-d78d-4cc9-80e7-7ccc36b556e1"),
        )

        sanh = Qualification.objects.create(
            category=medical_category,
            title="Sanitätshelfer",
            abbreviation="SanH",
            uuid=uuid.UUID("b1faab38-2e7c-4507-b753-06d1e653412d"),
        )
        sanh.included_qualifications.add(eh)

        rh = Qualification.objects.create(
            category=medical_category,
            title="Rettungshelfer",
            abbreviation="RH",
            uuid=uuid.UUID("de6c2449-a764-4c39-8f44-de276e99451b"),
        )
        rh.included_qualifications.add(sanh)

        rs = Qualification.objects.create(
            category=medical_category,
            title="Rettungssanitäter",
            abbreviation="RS",
            uuid=uuid.UUID("0b41fac6-ca9e-4b8a-82c5-849412187351"),
        )
        rs.included_qualifications.add(rh)

        ra = Qualification.objects.create(
            category=medical_category,
            title="Rettungsassistent",
            abbreviation="RA",
            uuid=uuid.UUID("2a24e395-fa02-4b2e-8215-7763b04d85c8"),
        )
        ra.included_qualifications.add(rs)

        nfs = Qualification.objects.create(
            category=medical_category,
            title="Notfallsanitäter",
            abbreviation="NFS",
            uuid=uuid.UUID("d114125b-7cf4-49e2-8908-f93e2f95dfb8"),
        )
        nfs.included_qualifications.add(ra)

        na = Qualification.objects.create(
            category=medical_category,
            title="Notarzt",
            abbreviation="NA",
            uuid=uuid.UUID("cb4f4ebc-3adf-4d32-a427-0ac0f686038a"),
        )

        driverslicense_category = QualificationCategory.objects.create(
            title=_("License"),
            uuid=uuid.UUID("a5669cc2-7444-4046-8c33-d8ee0bbf881b"),
        )

        l = Qualification.objects.create(
            category=driverslicense_category,
            title="Fahrerlaubnis Klasse L",
            abbreviation="Fe L",
            uuid=uuid.UUID("1c007860-48eb-4c86-a35b-45c6de903de4"),
        )

        t = Qualification.objects.create(
            category=driverslicense_category,
            title="Fahrerlaubnis Klasse T",
            abbreviation="Fe T",
            uuid=uuid.UUID("cc5ab73d-a1d6-4e95-93d2-aaf6c51ae91f"),
        )

        am = Qualification.objects.create(
            category=driverslicense_category,
            title="Fahrerlaubnis Klasse AM",
            abbreviation="Fe AM",
            uuid=uuid.UUID("f96bdfef-f1b8-46aa-8755-e506075ebc88"),
        )

        a1 = Qualification.objects.create(
            category=driverslicense_category,
            title="Fahrerlaubnis Klasse A1",
            abbreviation="Fe A1",
            uuid=uuid.UUID("d7e9da8f-8386-4e7d-8706-62103bfe78f1"),
        )
        a1.included_qualifications.add(am)

        a2 = Qualification.objects.create(
            category=driverslicense_category,
            title="Fahrerlaubnis Klasse A2",
            abbreviation="Fe A2",
            uuid=uuid.UUID("fe06530c-8216-40a0-81dd-2beaa8d803e1"),
        )
        a2.included_qualifications.add(a1)

        a = Qualification.objects.create(
            category=driverslicense_category,
            title="Fahrerlaubnis Klasse A",
            abbreviation="Fe A",
            uuid=uuid.UUID("4ff00daa-1501-41c0-8d89-2e54cb43f293"),
        )
        a.included_qualifications.add(a2)

        b = Qualification.objects.create(
            category=driverslicense_category,
            title="Fahrerlaubnis Klasse B",
            abbreviation="Fe B",
            uuid=uuid.UUID("0715b687-877a-4fed-bde0-5ea06b1043fc"),
        )
        b.included_qualifications.add(am)
        b.included_qualifications.add(l)

        be = Qualification.objects.create(
            category=driverslicense_category,
            title="Fahrerlaubnis Klasse BE",
            abbreviation="Fe BE",
            uuid=uuid.UUID("31529f69-09d7-44cc-84f6-19fbfd949faa"),
        )
        be.included_qualifications.add(b)

        c1 = Qualification.objects.create(
            category=driverslicense_category,
            title="Fahrerlaubnis Klasse C1",
            abbreviation="Fe C1",
            uuid=uuid.UUID("c9898e6c-4ecf-4781-9c0a-884861e36a81"),
        )
        c1.included_qualifications.add(b)

        c = Qualification.objects.create(
            category=driverslicense_category,
            title="Fahrerlaubnis Klasse C",
            abbreviation="Fe C",
            uuid=uuid.UUID("2d2fc932-5206-4c2c-bb63-0bc579acea6f"),
        )
        c.included_qualifications.add(c1)

        c1e = Qualification.objects.create(
            category=driverslicense_category,
            title="Fahrerlaubnis Klasse C1E",
            abbreviation="Fe C1E",
            uuid=uuid.UUID("f5e3be89-59de-4b13-a92f-5949009f62d8"),
        )
        c1e.included_qualifications.add(c1)
        c1e.included_qualifications.add(be)

        ce = Qualification.objects.create(
            category=driverslicense_category,
            title="Fahrerlaubnis Klasse CE",
            abbreviation="Fe CE",
            uuid=uuid.UUID("736ca05a-7ff9-423a-9fa4-8b4641fde29c"),
        )
        ce.included_qualifications.add(c)
        ce.included_qualifications.add(c1e)

        d1 = Qualification.objects.create(
            category=driverslicense_category,
            title="Fahrerlaubnis Klasse D1",
            abbreviation="Fe D1",
            uuid=uuid.UUID("3df818c6-f4a7-400e-a5bf-64ed087f79ab"),
        )
        d1.included_qualifications.add(b)

        d = Qualification.objects.create(
            category=driverslicense_category,
            title="Fahrerlaubnis Klasse D",
            abbreviation="Fe D",
            uuid=uuid.UUID("3238a892-afa8-4e76-b9b9-87d8765c5b72"),
        )
        d.included_qualifications.add(d1)

        d1e = Qualification.objects.create(
            category=driverslicense_category,
            title="Fahrerlaubnis Klasse D1E",
            abbreviation="Fe D1E",
            uuid=uuid.UUID("5d0ab40e-fade-4d45-9882-0039118a2445"),
        )
        d1e.included_qualifications.add(d1)
        d1e.included_qualifications.add(be)

        de = Qualification.objects.create(
            category=driverslicense_category,
            title="Fahrerlaubnis Klasse DE",
            abbreviation="Fe DE",
            uuid=uuid.UUID("64495d88-3ec0-4bbe-bc2b-78b6e6cfbc25"),
        )
        de.included_qualifications.add(d)
        de.included_qualifications.add(d1e)


class Command(BaseCommand):
    help = "Load initial data from datasets."

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.datasets = {ds.identifier: ds(self) for ds in AVAILABLE_DATASET_CLASSES}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(
            title="choosing the dataset",
            metavar="DATASET_IDENTIFIER",
            dest="dataset_identifier",
            help="Which data set to load. Available choices are:",
            required=True,
        )
        for dataset in self.datasets.values():
            dataset_parser = subparsers.add_parser(dataset.identifier, help=dataset.description)
            dataset.add_arguments(dataset_parser)

    def handle(self, *args, **options):
        if UserProfile.objects.exists():
            self.stdout.write("WARNING! User objects already exist in your database.")
            if input("Are you sure you want to continue? (yes/no) ") != "yes":
                self.stdout.write("Aborting...")
                return

        identifier = options["dataset_identifier"]
        with transaction.atomic():
            self.datasets[identifier].create_objects(*args, **options)
        self.stdout.write(self.style.SUCCESS("Done."))



# Generated by Django 3.0.6 on 2020-08-29 23:16

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
        ("event_management", "0005_auto_20200829_2348"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="abstractparticipation",
            options={"base_manager_name": "objects"},
        ),
        migrations.AlterModelOptions(
            name="localparticipation",
            options={"base_manager_name": "objects"},
        ),
        migrations.AddField(
            model_name="abstractparticipation",
            name="polymorphic_ctype",
            field=models.ForeignKey(
                editable=False,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="polymorphic_event_management.abstractparticipation_set+",
                to="contenttypes.ContentType",
            ),
        ),
    ]



from django import forms
from django.template.loader import get_template
from django.utils.translation import gettext_lazy as _
from django_select2.forms import Select2MultipleWidget

from ephios.event_management.models import AbstractParticipation
from ephios.event_management.signup import BaseSignupMethod, ParticipationError
from ephios.user_management.models import Qualification


class SimpleQualificationsRequiredSignupMethod(BaseSignupMethod):
    def __init__(self, shift):
        super().__init__(shift)
        if shift is not None:
            self.configuration.required_qualifications = Qualification.objects.filter(
                pk__in=self.configuration.required_qualification_ids
            )

    @property
    def signup_checkers(self):
        return super().signup_checkers + [self.check_qualification]

    @staticmethod
    def check_qualification(method, participant):
        if not participant.has_qualifications(method.configuration.required_qualifications):
            return ParticipationError(_("You are not qualified."))

    def get_configuration_fields(self):
        return {
            **super().get_configuration_fields(),
            "required_qualification_ids": {
                "formfield": forms.ModelMultipleChoiceField(
                    label=_("Required Qualifications"),
                    queryset=Qualification.objects.all(),
                    widget=Select2MultipleWidget,
                    required=False,
                ),
                "default": [],
                "publish_with_label": _("Required Qualification"),
                "format": lambda ids: ", ".join(
                    Qualification.objects.filter(id__in=ids).values_list("title", flat=True)
                ),
            },
        }


class InstantConfirmationSignupMethod(SimpleQualificationsRequiredSignupMethod):
    slug = "instant_confirmation"
    verbose_name = _("Instant Confirmation")
    description = _("""This method instantly confirms every signup after it was requested.""")

    @property
    def signup_checkers(self):
        return super().signup_checkers + [self.check_maximum_number_of_participants]

    @staticmethod
    def check_maximum_number_of_participants(method, participant):
        if method.configuration.maximum_number_of_participants is not None:
            current_count = AbstractParticipation.objects.filter(
                shift=method.shift, state=AbstractParticipation.States.CONFIRMED
            ).count()
            if current_count >= method.configuration.maximum_number_of_participants:
                return ParticipationError(_("The maximum number of participants is reached."))

    def get_configuration_fields(self):
        return {
            **super().get_configuration_fields(),
            "maximum_number_of_participants": {
                "formfield": forms.IntegerField(min_value=1, required=False),
                "default": None,
                "publish_with_label": _("Maximum number of participants"),
            },
        }

    def render_shift_state(self, request):
        return get_template("basesignup/signup_instant_state.html").render({"shift": self.shift})

    def perform_signup(self, participant, **kwargs):
        participation = super().perform_signup(participant, **kwargs)
        participation.state = AbstractParticipation.States.CONFIRMED
        participation.save()
        return participation



from django.apps import AppConfig


class PluginConfig(AppConfig):
    name = "ephios.plugins.pages"

    def ready(self):
        from ephios.plugins.pages import signals


default_app_config = "ephios.plugins.pages.PluginConfig"



from django.contrib import admin
from django.contrib.auth.admin import UserAdmin as BaseUserAdmin

from ephios.user_management.forms import UserChangeForm, UserCreationForm
from ephios.user_management.models import (
    Qualification,
    QualificationCategory,
    QualificationGrant,
    UserProfile,
)


class UserAdmin(BaseUserAdmin):
    # The forms to add and change user instances
    form = UserChangeForm
    add_form = UserCreationForm

    list_display = ("email", "first_name", "last_name", "is_staff")
    fieldsets = (
        (None, {"fields": ("email", "password")}),
        (
            "Personal info",
            {
                "fields": (
                    "first_name",
                    "last_name",
                    "date_of_birth",
                    "phone",
                )
            },
        ),
        ("Permissions", {"fields": ("is_staff", "is_superuser", "user_permissions")}),
    )

    # add_fieldsets is not a standard ModelAdmin attribute. UserAdmin
    # overrides get_fieldsets to use this attribute when creating a user.
    add_fieldsets = (
        (
            None,
            {
                "classes": ("wide",),
                "fields": (
                    "email",
                    "password",
                    "password_validation",
                    "first_name",
                    "last_name",
                    "date_of_birth",
                    "phone",
                ),
            },
        ),
    )
    search_fields = ("email",)
    ordering = ("email",)
    filter_horizontal = ()


admin.site.register(UserProfile, UserAdmin)
admin.site.register(Qualification)
admin.site.register(QualificationGrant)
admin.site.register(QualificationCategory)



# Generated by Django 3.1.1 on 2020-09-25 14:54

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    replaces = [
        ("event_management", "0001_initial"),
        ("event_management", "0002_event_active"),
        ("event_management", "0003_auto_20200825_1351"),
        ("event_management", "0004_auto_20200829_1535"),
        ("event_management", "0005_auto_20200829_2348"),
        ("event_management", "0006_auto_20200830_0116"),
        ("event_management", "0007_auto_20200902_0016"),
        ("event_management", "0008_event_mail_updates"),
        ("event_management", "0009_auto_20200921_2155"),
    ]

    initial = True

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="EventSeries",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="EventType",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("title", models.CharField(max_length=254, verbose_name="title")),
                (
                    "can_grant_qualification",
                    models.BooleanField(verbose_name="can grant qualification"),
                ),
            ],
            options={
                "verbose_name": "event type",
                "verbose_name_plural": "event types",
            },
        ),
        migrations.CreateModel(
            name="Event",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("title", models.CharField(max_length=254, verbose_name="title")),
                (
                    "description",
                    models.TextField(blank=True, null=True, verbose_name="description"),
                ),
                ("location", models.CharField(max_length=254, verbose_name="location")),
                (
                    "series",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="event_management.eventseries",
                    ),
                ),
                (
                    "type",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="event_management.eventtype",
                        verbose_name="event type",
                    ),
                ),
                ("active", models.BooleanField(default=False)),
                (
                    "mail_updates",
                    models.BooleanField(default=True, verbose_name="send updates via mail"),
                ),
            ],
            options={
                "verbose_name": "event",
                "verbose_name_plural": "events",
                "permissions": [("view_past_event", "Can view past events")],
            },
        ),
        migrations.CreateModel(
            name="Shift",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("meeting_time", models.DateTimeField(verbose_name="meeting time")),
                ("start_time", models.DateTimeField(verbose_name="start time")),
                ("end_time", models.DateTimeField(verbose_name="end time")),
                ("signup_method_slug", models.SlugField(verbose_name="signup method")),
                ("signup_configuration", models.JSONField(default=dict)),
                (
                    "event",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="shifts",
                        to="event_management.event",
                        verbose_name="shifts",
                    ),
                ),
            ],
            options={
                "verbose_name": "shift",
                "verbose_name_plural": "shifts",
            },
        ),
        migrations.CreateModel(
            name="AbstractParticipation",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                (
                    "state",
                    models.IntegerField(
                        choices=[
                            (0, "requested"),
                            (1, "confirmed"),
                            (2, "declined by user"),
                            (3, "rejected by responsible"),
                        ],
                        default=0,
                        verbose_name="state",
                    ),
                ),
                (
                    "shift",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="event_management.shift",
                        verbose_name="shift",
                    ),
                ),
                (
                    "polymorphic_ctype",
                    models.ForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="polymorphic_event_management.abstractparticipation_set+",
                        to="contenttypes.contenttype",
                    ),
                ),
            ],
            options={
                "base_manager_name": "objects",
            },
        ),
        migrations.CreateModel(
            name="LocalParticipation",
            fields=[
                (
                    "abstractparticipation_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="event_management.abstractparticipation",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL
                    ),
                ),
            ],
            options={
                "base_manager_name": "objects",
            },
            bases=("event_management.abstractparticipation",),
        ),
    ]



from django.apps import AppConfig


class ExtraConfig(AppConfig):
    name = "ephios.extra"

    def ready(self):
        from ephios.extra import signals


default_app_config = "ephios.extra.ExtraConfig"



# Generated by Django 3.0.6 on 2020-08-24 11:43

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("event_management", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="event",
            name="active",
            field=models.BooleanField(default=False),
        ),
    ]



from django.contrib import admin
from guardian.admin import GuardedModelAdmin

from ephios.event_management.models import Event, EventType, LocalParticipation, Shift

admin.site.register(Shift)
admin.site.register(Event, GuardedModelAdmin)
admin.site.register(EventType)
admin.site.register(LocalParticipation)



from django.contrib.auth import get_user_model
from django.shortcuts import get_object_or_404
from django_ical.views import ICalFeed
from guardian.shortcuts import get_users_with_perms
from icalendar import vCalAddress

from ephios.event_management.models import AbstractParticipation, Shift
from ephios.settings import SITE_URL


class EventFeed(ICalFeed):
    file_name = "events.ics"
    timezone = "UTC"
    product_id = "-//ephios//ephios//EN"

    def items(self):
        return Shift.objects.all().filter(event__active=True).order_by("meeting_time")

    def item_title(self, item):
        return item.event.title

    def item_description(self, item):
        return item.event.description

    def item_start_datetime(self, item):
        return item.meeting_time

    def item_end_datetime(self, item):
        return item.end_time

    def item_link(self, item):
        return item.event.get_absolute_url()

    def item_location(self, item):
        return item.event.location

    def item_guid(self, item):
        return f"{item.pk}@{SITE_URL}"

    def item_organizer(self, item):
        user = get_users_with_perms(item.event, only_with_perms_in=["change_event"]).first()
        email = user.email if user else ""
        return vCalAddress(f"MAILTO:{email}")


class UserEventFeed(EventFeed):
    def __init__(self, user):
        super().__init__()
        self.user = user

    def items(self):
        return self.user.get_shifts(
            with_participation_state_in=[AbstractParticipation.States.CONFIRMED]
        )


def user_event_feed_view(request, *args, **kwargs):
    feed = UserEventFeed(get_object_or_404(get_user_model(), **kwargs))
    return feed(request, *args, **kwargs)



from django.apps import AppConfig


class PluginConfig(AppConfig):
    name = "ephios.plugins.basesignup"

    def ready(self):
        from ephios.plugins.basesignup import signals


default_app_config = "ephios.plugins.basesignup.PluginConfig"



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ephios',
 'ephios.event_management',
 'ephios.event_management.migrations',
 'ephios.event_management.templatetags',
 'ephios.extra',
 'ephios.extra.management',
 'ephios.extra.management.commands',
 'ephios.extra.templatetags',
 'ephios.plugins',
 'ephios.plugins.basesignup',
 'ephios.plugins.basesignup.signup',
 'ephios.plugins.pages',
 'ephios.plugins.pages.migrations',
 'ephios.user_management',
 'ephios.user_management.migrations']

package_data = \
{'': ['*'],
 'ephios': ['locale/de/LC_MESSAGES/*',
            'static/bootstrap/css/*',
            'static/bootstrap/js/*',
            'static/datatables/*',
            'static/datatables/DataTables-1.10.21/css/*',
            'static/datatables/DataTables-1.10.21/images/*',
            'static/datatables/DataTables-1.10.21/js/*',
            'static/datatables/Responsive-2.2.5/css/*',
            'static/datatables/Responsive-2.2.5/js/*',
            'static/ephios/css/*',
            'static/ephios/js/*',
            'static/fontawesome/css/*',
            'static/fontawesome/webfonts/*',
            'static/select2/css/*',
            'static/select2/js/*',
            'static/select2/js/i18n/*',
            'static/sortablejs/*',
            'templates/*',
            'templates/registration/*'],
 'ephios.event_management': ['templates/event_management/*',
                             'templates/event_management/fragments/*',
                             'templates/event_management/mails/*'],
 'ephios.plugins': ['basesignup/templates/basesignup/*',
                    'basesignup/templates/basesignup/requestconfirm_signup/*',
                    'pages/templates/pages/*'],
 'ephios.user_management': ['templates/user_management/*']}

install_requires = \
['bleach>=3.2.1,<4.0.0',
 'django-bootstrap4>=2.2.0,<3.0.0',
 'django-csp>=3.7,<4.0',
 'django-environ>=0.4.5,<0.5.0',
 'django-formset-js-improved>=0.5.0,<0.6.0',
 'django-guardian>=2.3.0,<3.0.0',
 'django-ical>=1.7.1,<2.0.0',
 'django-polymorphic>=3.0.0,<4.0.0',
 'django-select2>=7.4.2,<8.0.0',
 'django>=3.1,<4.0',
 'markdown>=3.2.2,<4.0.0',
 'reportlab>=3.5.51,<4.0.0']

extras_require = \
{'mysql': ['mysqlclient>=2.0.1,<3.0.0'], 'pgsql': ['psycopg2>=2.8.6,<3.0.0']}

setup_kwargs = {
    'name': 'ephios',
    'version': '0.1.3',
    'description': 'ephios is a tool to manage shifts for medical services.',
    'long_description': '![ephios](https://github.com/ephios-dev/ephios/workflows/ephios/badge.svg)\n# ephios\nephios is a tool to manage shifts for medical services.\n\n## Development setup\n\nTo set up a development version on your local machine, you need to execute the following steps:\n1. Check out repository and cd to it\n2. Set up a virtualenv for the project with Python >=3.8 and activate it\n3. Install poetry (if not already installed): `curl -sSL https://raw.githubusercontent.com/python-poetry/poetry/master/get-poetry.py | python`\n4. Install dependencies with `poetry install`\n5. Create env file with `cp .env.example .env`\n6. Migrate the database with `python manage.py migrate`\n7. Compile translations with `python manage.py compilemessages`\n8. Load data for testing with `python manage.py setupdata debug`\n9. Start the development server with `python manage.py runserver`\n10. Open your web browser, visit `http://localhost:8000` and log in with the default credentials (user `admin@localhost` and password `admin`)\n\nBefore committing, make sure to lint your changes with `black .`. You can also check the [IDE integration](https://github.com/psf/black#editor-integration) or install a pre-commit hook with `pre-commit install` (recommended). You also need to to test the code with `pytest`.\n',
    'author': 'Julian Baumann',
    'author_email': 'julian@ephios.de',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://ephios.de',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from django_select2.forms import ModelSelect2MultipleWidget, ModelSelect2Widget

from ephios.user_management.models import UserProfile


class UserProfileWidget(ModelSelect2Widget):
    search_fields = ["email__icontains", "first_name__icontains", "last_name__icontains"]

    def label_from_instance(self, obj):
        return obj.get_full_name()


class MultiUserProfileWidget(ModelSelect2MultipleWidget):
    model = UserProfile

    search_fields = ["email__icontains", "first_name__icontains", "last_name__icontains"]

    def label_from_instance(self, obj):
        return obj.get_full_name()



# Generated by Django 3.0.6 on 2020-08-29 13:35

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("user_management", "0003_userprofile_calendar_token"),
    ]

    operations = [
        migrations.CreateModel(
            name="QualificationCategory",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("uuid", models.UUIDField(unique=True)),
                ("title", models.CharField(max_length=254)),
            ],
        ),
        migrations.RemoveField(
            model_name="qualification",
            name="track",
        ),
        migrations.RemoveField(
            model_name="qualificationgrant",
            name="expiration_date",
        ),
        migrations.AddField(
            model_name="qualification",
            name="included_qualifications",
            field=models.ManyToManyField(
                related_name="included_in_set", to="user_management.Qualification"
            ),
        ),
        migrations.AddField(
            model_name="qualification",
            name="uuid",
            field=models.UUIDField(default=None, unique=True),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="qualificationgrant",
            name="expires",
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name="qualificationgrant",
            name="qualification",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="grants",
                to="user_management.Qualification",
            ),
        ),
        migrations.AlterField(
            model_name="qualificationgrant",
            name="user",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="qualification_grants",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
        migrations.DeleteModel(
            name="QualificationTrack",
        ),
        migrations.AddField(
            model_name="qualification",
            name="category",
            field=models.ForeignKey(
                default=None,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="qualifications",
                to="user_management.QualificationCategory",
            ),
            preserve_default=False,
        ),
    ]



# Generated by Django 3.0.6 on 2020-08-29 13:35

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("event_management", "0003_auto_20200825_1351"),
    ]

    operations = [
        migrations.AlterField(
            model_name="abstractparticipation",
            name="state",
            field=models.IntegerField(
                choices=[
                    (0, "requested"),
                    (1, "confirmed"),
                    (2, "declined by user"),
                    (3, "rejected by responsible"),
                ],
                default=0,
            ),
        ),
    ]



from django.contrib.auth.mixins import PermissionRequiredMixin
from django.contrib.auth.models import Group, Permission
from guardian.ctypes import get_content_type
from guardian.utils import get_group_obj_perms_model


class CustomPermissionRequiredMixin(PermissionRequiredMixin):
    """
    As of 2020-09-26, guardians permission mixin
    doesn't support the mode of operation we want, but Django's does:
    * Logged in users without permission get 403
    * not logged in users get redirected to login
    Therefore we patch Django's mixin to support object permissions
    """

    accept_global_perms = True

    def get_permission_object(self):
        if hasattr(self, "permission_object"):
            return self.permission_object
        return hasattr(self, "get_object") and self.get_object() or getattr(self, "object", None)

    def has_permission(self):
        user = self.request.user
        perms = self.get_permission_required()
        if self.accept_global_perms and all(user.has_perm(perm) for perm in perms):
            return True
        if (obj := self.get_permission_object()) is None:
            return False
        return all(user.has_perm(perm, obj) for perm in perms)


def get_groups_with_perms(obj, only_with_perms_in):
    ctype = get_content_type(obj)
    group_model = get_group_obj_perms_model(obj)

    group_rel_name = group_model.group.field.related_query_name()

    if group_model.objects.is_generic():
        group_filters = {
            "%s__content_type" % group_rel_name: ctype,
            "%s__object_pk" % group_rel_name: obj.pk,
        }
    else:
        group_filters = {"%s__content_object" % group_rel_name: obj}

    permission_ids = Permission.objects.filter(
        content_type=ctype, codename__in=only_with_perms_in
    ).values_list("id", flat=True)
    group_filters.update(
        {
            "%s__permission_id__in" % group_rel_name: permission_ids,
        }
    )
    return Group.objects.filter(**group_filters).distinct()



# Generated by Django 3.1.1 on 2020-09-25 14:40

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("user_management", "0007_auto_20200921_2155"),
    ]

    operations = [
        migrations.AlterField(
            model_name="qualificationgrant",
            name="qualification",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="grants",
                to="user_management.qualification",
                verbose_name="qualification",
            ),
        ),
        migrations.AlterField(
            model_name="userprofile",
            name="phone",
            field=models.CharField(
                blank=True, default="", max_length=254, verbose_name="phone number"
            ),
            preserve_default=False,
        ),
    ]



from datetime import datetime, timedelta

from django.contrib.auth.models import Group
from django.core.exceptions import ValidationError
from django.db.models import Q
from django.forms import DateField, ModelForm, ModelMultipleChoiceField, Select, TimeField
from django.utils.timezone import make_aware
from django.utils.translation import gettext as _
from django_select2.forms import Select2MultipleWidget
from guardian.shortcuts import (
    assign_perm,
    remove_perm,
    get_users_with_perms,
)

from ephios.event_management import signup
from ephios.event_management.models import Event, Shift
from ephios.extra.permissions import get_groups_with_perms
from ephios.extra.widgets import CustomDateInput, CustomTimeInput
from ephios.user_management.models import UserProfile
from ephios.user_management.widgets import MultiUserProfileWidget


class EventForm(ModelForm):
    visible_for = ModelMultipleChoiceField(
        queryset=Group.objects.none(),
        label=_("Visible for"),
        help_text=_("Select groups which the event shall be visible for."),
        widget=Select2MultipleWidget,
        required=False,
    )
    responsible_persons = ModelMultipleChoiceField(
        queryset=UserProfile.objects.all(),
        required=False,
        label=_("Responsible persons"),
        widget=MultiUserProfileWidget,
    )
    responsible_groups = ModelMultipleChoiceField(
        queryset=Group.objects.all(),
        required=False,
        label=_("Responsible groups"),
        widget=Select2MultipleWidget,
    )

    class Meta:
        model = Event
        fields = ["title", "description", "location", "type", "mail_updates"]

    def save(self, commit=True):
        event = super().save(commit)

        # delete existing permissions
        # (better implement https://github.com/django-guardian/django-guardian/issues/654)
        for group in get_groups_with_perms(
            event, only_with_perms_in=["view_event", "change_event"]
        ):
            remove_perm("view_event", group, event)
            remove_perm("change_event", group, event)
        for user in get_users_with_perms(event, only_with_perms_in=["view_event", "change_event"]):
            remove_perm("view_event", user, event)
            remove_perm("change_event", user, event)

        # assign designated permissions
        assign_perm(
            "view_event",
            Group.objects.filter(
                Q(id__in=self.cleaned_data["visible_for"])
                | Q(id__in=self.cleaned_data["responsible_groups"])
            ),
            event,
        )
        assign_perm("change_event", self.cleaned_data["responsible_groups"], event)
        assign_perm("view_event", self.cleaned_data["responsible_persons"], event)
        assign_perm("change_event", self.cleaned_data["responsible_persons"], event)

        return event


class ShiftForm(ModelForm):
    date = DateField(widget=CustomDateInput(format="%Y-%m-%d"))
    meeting_time = TimeField(widget=CustomTimeInput)
    start_time = TimeField(widget=CustomTimeInput)
    end_time = TimeField(widget=CustomTimeInput)

    field_order = ["date", "meeting_time", "start_time", "end_time", "signup_method_slug"]

    class Meta:
        model = Shift
        fields = ["meeting_time", "start_time", "end_time", "signup_method_slug"]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields["signup_method_slug"].widget = Select(
            choices=((method.slug, method.verbose_name) for method in signup.all_signup_methods())
        )

    def clean(self):
        cleaned_data = super().clean()
        if {"date", "meeting_time", "start_time", "end_time"} <= set(cleaned_data.keys()):
            cleaned_data["meeting_time"] = make_aware(
                datetime.combine(cleaned_data["date"], cleaned_data["meeting_time"])
            )
            cleaned_data["start_time"] = make_aware(
                datetime.combine(cleaned_data["date"], cleaned_data["start_time"])
            )
            cleaned_data["end_time"] = make_aware(
                datetime.combine(self.cleaned_data["date"], cleaned_data["end_time"])
            )
            if self.cleaned_data["end_time"] <= self.cleaned_data["start_time"]:
                cleaned_data["end_time"] = cleaned_data["end_time"] + timedelta(days=1)
            if not cleaned_data["meeting_time"] <= cleaned_data["start_time"]:
                raise ValidationError(_("Meeting time must not be after start time!"))
        return cleaned_data



# Generated by Django 3.0.9 on 2020-09-01 22:16

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("event_management", "0006_auto_20200830_0116"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="event",
            options={
                "permissions": [("view_past_event", "Can view events in the past")],
                "verbose_name": "event",
                "verbose_name_plural": "events",
            },
        ),
    ]



import urllib.parse

import bleach
import markdown
from bleach import DEFAULT_CALLBACKS
from bleach.linkifier import build_email_re, build_url_re
from django import template
from django.conf import settings
from django.core import signing
from django.urls import reverse
from django.utils.http import url_has_allowed_host_and_scheme
from django.utils.safestring import mark_safe

register = template.Library()

ALLOWED_TAGS = [
    "a",
    "abbr",
    "acronym",
    "b",
    "blockquote",
    "br",
    "code",
    "div",
    "em",
    "h1",
    "h2",
    "h3",
    "h4",
    "h5",
    "h6",
    "hr",
    "i",
    "li",
    "ol",
    "p",
    "pre",
    "span",
    "strong",
    "table",
    "tbody",
    "td",
    "th",
    "thead",
    "tr",
    "ul",
]


ALLOWED_ATTRIBUTES = {
    "a": ["href", "title", "class"],
    "abbr": ["title"],
    "acronym": ["title"],
    "table": ["width"],
    "td": ["width", "align"],
    "div": ["class"],
    "p": ["class"],
    "span": ["class", "title"],
}

ALLOWED_PROTOCOLS = ["http", "https", "mailto", "tel"]


def markdown_compile(source):
    extensions = ["markdown.extensions.sane_lists", "markdown.extensions.nl2br"]
    return bleach.clean(
        markdown.markdown(source, extensions=extensions),
        tags=ALLOWED_TAGS,
        attributes=ALLOWED_ATTRIBUTES,
        protocols=ALLOWED_PROTOCOLS,
    )


@register.filter
def rich_text(text: str, **kwargs):
    """
    Processes markdown and cleans HTML in a text input.
    """
    text = str(text)
    linker = bleach.Linker(parse_email=True)
    body_md = linker.linkify(markdown_compile(text))
    return mark_safe(body_md)



import secrets
import uuid
from datetime import date, datetime

import guardian.mixins
from django.contrib.auth import get_user_model
from django.contrib.auth.base_user import AbstractBaseUser, BaseUserManager
from django.contrib.auth.models import PermissionsMixin
from django.db import models
from django.db.models import (
    BooleanField,
    CharField,
    DateField,
    EmailField,
    Exists,
    F,
    ForeignKey,
    Max,
    Model,
    OuterRef,
    Q,
)
from django.utils import timezone
from django.utils.translation import gettext_lazy as _


class UserManager(BaseUserManager):
    def create_user(
        self,
        email,
        first_name,
        last_name,
        date_of_birth,
        password=None,
    ):
        user = self.model(
            email=email,
            first_name=first_name,
            last_name=last_name,
            date_of_birth=date_of_birth,
        )
        user.set_password(password)
        user.save()
        return user

    def create_superuser(
        self,
        email,
        first_name,
        last_name,
        date_of_birth,
        password=None,
    ):
        user = self.create_user(
            email=email,
            password=password,
            first_name=first_name,
            last_name=last_name,
            date_of_birth=date_of_birth,
        )
        user.is_superuser = True
        user.is_staff = True
        user.save()
        return user


class UserProfile(AbstractBaseUser, PermissionsMixin, guardian.mixins.GuardianUserMixin):
    email = EmailField(_("email address"), unique=True)
    is_active = BooleanField(default=True)
    is_staff = BooleanField(default=False)
    first_name = CharField(_("first name"), max_length=254)
    last_name = CharField(_("last name"), max_length=254)
    date_of_birth = DateField(_("date of birth"))
    phone = CharField(_("phone number"), max_length=254, blank=True)
    calendar_token = CharField(_("calendar token"), max_length=254, default=secrets.token_urlsafe)

    USERNAME_FIELD = "email"
    REQUIRED_FIELDS = [
        "first_name",
        "last_name",
        "date_of_birth",
    ]

    objects = UserManager()

    class Meta:
        verbose_name = _("user profile")
        verbose_name_plural = _("user profiles")

    def get_full_name(self):
        return self.first_name + " " + self.last_name

    def get_short_name(self):
        return self.first_name

    @property
    def age(self):
        today, born = date.today(), self.date_of_birth
        return today.year - born.year - ((today.month, today.day) < (born.month, born.day))

    @property
    def is_minor(self):
        return self.age < 18

    def as_participant(self):
        from ephios.event_management.signup import LocalUserParticipant

        return LocalUserParticipant(
            first_name=self.first_name,
            last_name=self.last_name,
            qualifications=self.qualifications,
            date_of_birth=self.date_of_birth,
            email=self.email if self.is_active else None,
            user=self,
        )

    @property
    def qualifications(self):
        return Qualification.objects.annotate(
            has_active_grant=Exists(
                QualificationGrant.objects.filter(user=self, qualification=OuterRef("pk")).filter(
                    Q(expires__gt=timezone.now()) | Q(expires__isnull=True)
                )
            ),
            expires=Max(F("grants__expires")),
        ).filter(has_active_grant=True)

    def get_shifts(self, with_participation_state_in):
        from ephios.event_management.models import Shift

        shift_ids = self.localparticipation_set.filter(
            state__in=with_participation_state_in
        ).values_list("shift", flat=True)
        return Shift.objects.filter(pk__in=shift_ids)


class QualificationCategory(Model):
    uuid = models.UUIDField(unique=True, default=uuid.uuid4)
    title = CharField(_("title"), max_length=254)

    class Meta:
        verbose_name = _("qualification track")
        verbose_name_plural = _("qualification tracks")

    def __str__(self):
        return self.title


class Qualification(Model):
    uuid = models.UUIDField(unique=True, default=uuid.uuid4)
    title = CharField(_("title"), max_length=254)
    abbreviation = CharField(max_length=254)
    category = ForeignKey(
        QualificationCategory,
        on_delete=models.CASCADE,
        related_name="qualifications",
        verbose_name=_("category"),
    )
    included_qualifications = models.ManyToManyField(
        "self", related_name="included_by", symmetrical=False
    )

    def __eq__(self, other):
        return self.uuid == other.uuid if other else False

    def __hash__(self):
        return hash(self.uuid)

    class Meta:
        verbose_name = _("qualification")
        verbose_name_plural = _("qualifications")

    def __str__(self):
        return self.title


class QualificationGrant(Model):
    qualification = ForeignKey(
        Qualification,
        on_delete=models.CASCADE,
        verbose_name=_("qualification"),
        related_name="grants",
    )
    user = ForeignKey(get_user_model(), on_delete=models.CASCADE, verbose_name=_("user profile"))
    expires = models.DateTimeField(_("expiration date"), blank=True, null=True)

    def __str__(self):
        return f"{self.qualification!s}, {self.user!s}"



from django.dispatch import receiver
from django.urls import reverse

from ephios.extra.signals import footer_link
from ephios.plugins.pages.models import Page


@receiver(footer_link, dispatch_uid="ephios.plugins.pages.signals.pages_footer_links")
def pages_footer_links(sender, request, **kwargs):
    pages = Page.objects.filter(show_in_footer=True)
    if request.user.is_anonymous:
        pages = pages.filter(publicly_visible=True)
    return {page.title: reverse("pages:view_page", kwargs=dict(slug=page.slug)) for page in pages}
